# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from scaleway_core.utils import (
    OneOfPossibility,
    resolve_one_of,
)
from dateutil import parser
from .types import (
    AclActionRedirectRedirectType,
    AclActionType,
    AclHttpFilter,
    ForwardPortAlgorithm,
    OnMarkedDownAction,
    Protocol,
    ProxyProtocol,
    SSLCompatibilityLevel,
    StickySessionsType,
    Acl,
    AclAction,
    AclActionRedirect,
    AclMatch,
    AclSpec,
    Backend,
    BackendServerStats,
    Certificate,
    CreateCertificateRequestCustomCertificate,
    CreateCertificateRequestLetsencryptConfig,
    Frontend,
    HealthCheck,
    HealthCheckHttpConfig,
    HealthCheckHttpsConfig,
    HealthCheckLdapConfig,
    HealthCheckMysqlConfig,
    HealthCheckPgsqlConfig,
    HealthCheckRedisConfig,
    HealthCheckTcpConfig,
    Instance,
    Ip,
    Lb,
    LbStats,
    LbType,
    ListAclResponse,
    ListBackendStatsResponse,
    ListBackendsResponse,
    ListCertificatesResponse,
    ListFrontendsResponse,
    ListIpsResponse,
    ListLbPrivateNetworksResponse,
    ListLbTypesResponse,
    ListLbsResponse,
    ListRoutesResponse,
    ListSubscriberResponse,
    PrivateNetwork,
    PrivateNetworkDHCPConfig,
    PrivateNetworkIpamConfig,
    PrivateNetworkStaticConfig,
    Route,
    RouteMatch,
    SetAclsResponse,
    Subscriber,
    SubscriberEmailConfig,
    SubscriberWebhookConfig,
    CreateLbRequest,
    UpdateLbRequest,
    MigrateLbRequest,
    CreateIpRequest,
    UpdateIpRequest,
    CreateBackendRequest,
    UpdateBackendRequest,
    AddBackendServersRequest,
    RemoveBackendServersRequest,
    SetBackendServersRequest,
    UpdateHealthCheckRequest,
    CreateFrontendRequest,
    UpdateFrontendRequest,
    CreateRouteRequest,
    UpdateRouteRequest,
    CreateAclRequest,
    UpdateAclRequest,
    CreateCertificateRequest,
    UpdateCertificateRequest,
    CreateSubscriberRequest,
    UpdateSubscriberRequest,
    SubscribeToLbRequest,
    AttachPrivateNetworkRequest,
    ZonedApiCreateLbRequest,
    ZonedApiUpdateLbRequest,
    ZonedApiMigrateLbRequest,
    ZonedApiCreateIpRequest,
    ZonedApiUpdateIpRequest,
    ZonedApiCreateBackendRequest,
    ZonedApiUpdateBackendRequest,
    ZonedApiAddBackendServersRequest,
    ZonedApiRemoveBackendServersRequest,
    ZonedApiSetBackendServersRequest,
    ZonedApiUpdateHealthCheckRequest,
    ZonedApiCreateFrontendRequest,
    ZonedApiUpdateFrontendRequest,
    ZonedApiCreateRouteRequest,
    ZonedApiUpdateRouteRequest,
    ZonedApiCreateAclRequest,
    ZonedApiUpdateAclRequest,
    ZonedApiSetAclsRequest,
    ZonedApiCreateCertificateRequest,
    ZonedApiUpdateCertificateRequest,
    ZonedApiCreateSubscriberRequest,
    ZonedApiUpdateSubscriberRequest,
    ZonedApiSubscribeToLbRequest,
    ZonedApiAttachPrivateNetworkRequest,
)


def unmarshal_SubscriberEmailConfig(data: Any) -> SubscriberEmailConfig:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'SubscriberEmailConfig' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("email", None)
    args["email"] = field

    return SubscriberEmailConfig(**args)


def unmarshal_SubscriberWebhookConfig(data: Any) -> SubscriberWebhookConfig:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'SubscriberWebhookConfig' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("uri", None)
    args["uri"] = field

    return SubscriberWebhookConfig(**args)


def unmarshal_HealthCheckHttpConfig(data: Any) -> HealthCheckHttpConfig:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'HealthCheckHttpConfig' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("code", None)
    args["code"] = field

    field = data.get("host_header", None)
    args["host_header"] = field

    field = data.get("method", None)
    args["method"] = field

    field = data.get("uri", None)
    args["uri"] = field

    return HealthCheckHttpConfig(**args)


def unmarshal_HealthCheckHttpsConfig(data: Any) -> HealthCheckHttpsConfig:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'HealthCheckHttpsConfig' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("code", None)
    args["code"] = field

    field = data.get("host_header", None)
    args["host_header"] = field

    field = data.get("method", None)
    args["method"] = field

    field = data.get("sni", None)
    args["sni"] = field

    field = data.get("uri", None)
    args["uri"] = field

    return HealthCheckHttpsConfig(**args)


def unmarshal_HealthCheckLdapConfig(data: Any) -> HealthCheckLdapConfig:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'HealthCheckLdapConfig' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    return HealthCheckLdapConfig(**args)


def unmarshal_HealthCheckMysqlConfig(data: Any) -> HealthCheckMysqlConfig:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'HealthCheckMysqlConfig' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("user", None)
    args["user"] = field

    return HealthCheckMysqlConfig(**args)


def unmarshal_HealthCheckPgsqlConfig(data: Any) -> HealthCheckPgsqlConfig:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'HealthCheckPgsqlConfig' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("user", None)
    args["user"] = field

    return HealthCheckPgsqlConfig(**args)


def unmarshal_HealthCheckRedisConfig(data: Any) -> HealthCheckRedisConfig:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'HealthCheckRedisConfig' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    return HealthCheckRedisConfig(**args)


def unmarshal_HealthCheckTcpConfig(data: Any) -> HealthCheckTcpConfig:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'HealthCheckTcpConfig' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    return HealthCheckTcpConfig(**args)


def unmarshal_Instance(data: Any) -> Instance:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Instance' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("ip_address", None)
    args["ip_address"] = field

    field = data.get("region", None)
    args["region"] = field

    field = data.get("status", None)
    args["status"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("zone", None)
    args["zone"] = field

    return Instance(**args)


def unmarshal_Ip(data: Any) -> Ip:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Ip' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    args["id"] = field

    field = data.get("ip_address", None)
    args["ip_address"] = field

    field = data.get("lb_id", None)
    args["lb_id"] = field

    field = data.get("organization_id", None)
    args["organization_id"] = field

    field = data.get("project_id", None)
    args["project_id"] = field

    field = data.get("region", None)
    args["region"] = field

    field = data.get("reverse", None)
    args["reverse"] = field

    field = data.get("zone", None)
    args["zone"] = field

    return Ip(**args)


def unmarshal_Subscriber(data: Any) -> Subscriber:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Subscriber' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("email_config", None)
    args["email_config"] = (
        unmarshal_SubscriberEmailConfig(field) if field is not None else None
    )

    field = data.get("id", None)
    args["id"] = field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("webhook_config", None)
    args["webhook_config"] = (
        unmarshal_SubscriberWebhookConfig(field) if field is not None else None
    )

    return Subscriber(**args)


def unmarshal_HealthCheck(data: Any) -> HealthCheck:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'HealthCheck' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("check_delay", None)
    args["check_delay"] = field

    field = data.get("check_max_retries", None)
    args["check_max_retries"] = field

    field = data.get("check_send_proxy", None)
    args["check_send_proxy"] = field

    field = data.get("check_timeout", None)
    args["check_timeout"] = field

    field = data.get("http_config", None)
    args["http_config"] = (
        unmarshal_HealthCheckHttpConfig(field) if field is not None else None
    )

    field = data.get("https_config", None)
    args["https_config"] = (
        unmarshal_HealthCheckHttpsConfig(field) if field is not None else None
    )

    field = data.get("ldap_config", None)
    args["ldap_config"] = (
        unmarshal_HealthCheckLdapConfig(field) if field is not None else None
    )

    field = data.get("mysql_config", None)
    args["mysql_config"] = (
        unmarshal_HealthCheckMysqlConfig(field) if field is not None else None
    )

    field = data.get("pgsql_config", None)
    args["pgsql_config"] = (
        unmarshal_HealthCheckPgsqlConfig(field) if field is not None else None
    )

    field = data.get("port", None)
    args["port"] = field

    field = data.get("redis_config", None)
    args["redis_config"] = (
        unmarshal_HealthCheckRedisConfig(field) if field is not None else None
    )

    field = data.get("tcp_config", None)
    args["tcp_config"] = (
        unmarshal_HealthCheckTcpConfig(field) if field is not None else None
    )

    field = data.get("transient_check_delay", None)
    args["transient_check_delay"] = field

    return HealthCheck(**args)


def unmarshal_Lb(data: Any) -> Lb:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Lb' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("backend_count", None)
    args["backend_count"] = field

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("description", None)
    args["description"] = field

    field = data.get("frontend_count", None)
    args["frontend_count"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("instances", None)
    args["instances"] = (
        [unmarshal_Instance(v) for v in field] if field is not None else None
    )

    field = data.get("ip", None)
    args["ip"] = [unmarshal_Ip(v) for v in field] if field is not None else None

    field = data.get("name", None)
    args["name"] = field

    field = data.get("organization_id", None)
    args["organization_id"] = field

    field = data.get("private_network_count", None)
    args["private_network_count"] = field

    field = data.get("project_id", None)
    args["project_id"] = field

    field = data.get("region", None)
    args["region"] = field

    field = data.get("route_count", None)
    args["route_count"] = field

    field = data.get("ssl_compatibility_level", None)
    args["ssl_compatibility_level"] = field

    field = data.get("status", None)
    args["status"] = field

    field = data.get("subscriber", None)
    args["subscriber"] = unmarshal_Subscriber(field) if field is not None else None

    field = data.get("tags", None)
    args["tags"] = field

    field = data.get("type", None)
    args["type_"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("zone", None)
    args["zone"] = field

    return Lb(**args)


def unmarshal_AclActionRedirect(data: Any) -> AclActionRedirect:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'AclActionRedirect' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("code", None)
    args["code"] = field

    field = data.get("target", None)
    args["target"] = field

    field = data.get("type", None)
    args["type_"] = field

    return AclActionRedirect(**args)


def unmarshal_Backend(data: Any) -> Backend:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Backend' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("failover_host", None)
    args["failover_host"] = field

    field = data.get("forward_port", None)
    args["forward_port"] = field

    field = data.get("forward_port_algorithm", None)
    args["forward_port_algorithm"] = field

    field = data.get("forward_protocol", None)
    args["forward_protocol"] = field

    field = data.get("health_check", None)
    args["health_check"] = unmarshal_HealthCheck(field) if field is not None else None

    field = data.get("id", None)
    args["id"] = field

    field = data.get("ignore_ssl_server_verify", None)
    args["ignore_ssl_server_verify"] = field

    field = data.get("lb", None)
    args["lb"] = unmarshal_Lb(field) if field is not None else None

    field = data.get("max_connections", None)
    args["max_connections"] = field

    field = data.get("max_retries", None)
    args["max_retries"] = field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("on_marked_down_action", None)
    args["on_marked_down_action"] = field

    field = data.get("pool", None)
    args["pool"] = field

    field = data.get("proxy_protocol", None)
    args["proxy_protocol"] = field

    field = data.get("redispatch_attempt_count", None)
    args["redispatch_attempt_count"] = field

    field = data.get("send_proxy_v2", None)
    args["send_proxy_v2"] = field

    field = data.get("ssl_bridging", None)
    args["ssl_bridging"] = field

    field = data.get("sticky_sessions", None)
    args["sticky_sessions"] = field

    field = data.get("sticky_sessions_cookie_name", None)
    args["sticky_sessions_cookie_name"] = field

    field = data.get("timeout_connect", None)
    args["timeout_connect"] = field

    field = data.get("timeout_queue", None)
    args["timeout_queue"] = field

    field = data.get("timeout_server", None)
    args["timeout_server"] = field

    field = data.get("timeout_tunnel", None)
    args["timeout_tunnel"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return Backend(**args)


def unmarshal_Certificate(data: Any) -> Certificate:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Certificate' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("common_name", None)
    args["common_name"] = field

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("fingerprint", None)
    args["fingerprint"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("lb", None)
    args["lb"] = unmarshal_Lb(field) if field is not None else None

    field = data.get("name", None)
    args["name"] = field

    field = data.get("not_valid_after", None)
    args["not_valid_after"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("not_valid_before", None)
    args["not_valid_before"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("status", None)
    args["status"] = field

    field = data.get("status_details", None)
    args["status_details"] = field

    field = data.get("subject_alternative_name", None)
    args["subject_alternative_name"] = field

    field = data.get("type", None)
    args["type_"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return Certificate(**args)


def unmarshal_AclAction(data: Any) -> AclAction:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'AclAction' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("redirect", None)
    args["redirect"] = unmarshal_AclActionRedirect(field) if field is not None else None

    field = data.get("type", None)
    args["type_"] = field

    return AclAction(**args)


def unmarshal_AclMatch(data: Any) -> AclMatch:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'AclMatch' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("http_filter", None)
    args["http_filter"] = field

    field = data.get("http_filter_option", None)
    args["http_filter_option"] = field

    field = data.get("http_filter_value", None)
    args["http_filter_value"] = field

    field = data.get("invert", None)
    args["invert"] = field

    field = data.get("ip_subnet", None)
    args["ip_subnet"] = field

    return AclMatch(**args)


def unmarshal_Frontend(data: Any) -> Frontend:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Frontend' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("backend", None)
    args["backend"] = unmarshal_Backend(field) if field is not None else None

    field = data.get("certificate", None)
    args["certificate"] = unmarshal_Certificate(field) if field is not None else None

    field = data.get("certificate_ids", None)
    args["certificate_ids"] = field

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("enable_http3", None)
    args["enable_http3"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("inbound_port", None)
    args["inbound_port"] = field

    field = data.get("lb", None)
    args["lb"] = unmarshal_Lb(field) if field is not None else None

    field = data.get("name", None)
    args["name"] = field

    field = data.get("timeout_client", None)
    args["timeout_client"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return Frontend(**args)


def unmarshal_PrivateNetworkDHCPConfig(data: Any) -> PrivateNetworkDHCPConfig:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'PrivateNetworkDHCPConfig' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("ip_id", None)
    args["ip_id"] = field

    return PrivateNetworkDHCPConfig(**args)


def unmarshal_PrivateNetworkIpamConfig(data: Any) -> PrivateNetworkIpamConfig:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'PrivateNetworkIpamConfig' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    return PrivateNetworkIpamConfig(**args)


def unmarshal_PrivateNetworkStaticConfig(data: Any) -> PrivateNetworkStaticConfig:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'PrivateNetworkStaticConfig' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("ip_address", None)
    args["ip_address"] = field

    return PrivateNetworkStaticConfig(**args)


def unmarshal_RouteMatch(data: Any) -> RouteMatch:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'RouteMatch' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("host_header", None)
    args["host_header"] = field

    field = data.get("sni", None)
    args["sni"] = field

    return RouteMatch(**args)


def unmarshal_Acl(data: Any) -> Acl:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Acl' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("action", None)
    args["action"] = unmarshal_AclAction(field) if field is not None else None

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("description", None)
    args["description"] = field

    field = data.get("frontend", None)
    args["frontend"] = unmarshal_Frontend(field) if field is not None else None

    field = data.get("id", None)
    args["id"] = field

    field = data.get("index", None)
    args["index"] = field

    field = data.get("match", None)
    args["match"] = unmarshal_AclMatch(field) if field is not None else None

    field = data.get("name", None)
    args["name"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return Acl(**args)


def unmarshal_BackendServerStats(data: Any) -> BackendServerStats:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'BackendServerStats' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("backend_id", None)
    args["backend_id"] = field

    field = data.get("instance_id", None)
    args["instance_id"] = field

    field = data.get("ip", None)
    args["ip"] = field

    field = data.get("last_health_check_status", None)
    args["last_health_check_status"] = field

    field = data.get("server_state", None)
    args["server_state"] = field

    field = data.get("server_state_changed_at", None)
    args["server_state_changed_at"] = (
        parser.isoparse(field) if type(field) is str else field
    )

    return BackendServerStats(**args)


def unmarshal_LbType(data: Any) -> LbType:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'LbType' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("description", None)
    args["description"] = field

    field = data.get("name", None)
    args["name"] = field

    field = data.get("region", None)
    args["region"] = field

    field = data.get("stock_status", None)
    args["stock_status"] = field

    field = data.get("zone", None)
    args["zone"] = field

    return LbType(**args)


def unmarshal_PrivateNetwork(data: Any) -> PrivateNetwork:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'PrivateNetwork' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("dhcp_config", None)
    args["dhcp_config"] = (
        unmarshal_PrivateNetworkDHCPConfig(field) if field is not None else None
    )

    field = data.get("ipam_config", None)
    args["ipam_config"] = (
        unmarshal_PrivateNetworkIpamConfig(field) if field is not None else None
    )

    field = data.get("lb", None)
    args["lb"] = unmarshal_Lb(field) if field is not None else None

    field = data.get("private_network_id", None)
    args["private_network_id"] = field

    field = data.get("static_config", None)
    args["static_config"] = (
        unmarshal_PrivateNetworkStaticConfig(field) if field is not None else None
    )

    field = data.get("status", None)
    args["status"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return PrivateNetwork(**args)


def unmarshal_Route(data: Any) -> Route:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Route' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("backend_id", None)
    args["backend_id"] = field

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("frontend_id", None)
    args["frontend_id"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("match", None)
    args["match"] = unmarshal_RouteMatch(field) if field is not None else None

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return Route(**args)


def unmarshal_LbStats(data: Any) -> LbStats:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'LbStats' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("backend_servers_stats", None)
    args["backend_servers_stats"] = (
        [unmarshal_BackendServerStats(v) for v in field] if field is not None else None
    )

    return LbStats(**args)


def unmarshal_ListAclResponse(data: Any) -> ListAclResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListAclResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("acls", None)
    args["acls"] = [unmarshal_Acl(v) for v in field] if field is not None else None

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListAclResponse(**args)


def unmarshal_ListBackendStatsResponse(data: Any) -> ListBackendStatsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListBackendStatsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("backend_servers_stats", None)
    args["backend_servers_stats"] = (
        [unmarshal_BackendServerStats(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListBackendStatsResponse(**args)


def unmarshal_ListBackendsResponse(data: Any) -> ListBackendsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListBackendsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("backends", None)
    args["backends"] = (
        [unmarshal_Backend(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListBackendsResponse(**args)


def unmarshal_ListCertificatesResponse(data: Any) -> ListCertificatesResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListCertificatesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("certificates", None)
    args["certificates"] = (
        [unmarshal_Certificate(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListCertificatesResponse(**args)


def unmarshal_ListFrontendsResponse(data: Any) -> ListFrontendsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListFrontendsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("frontends", None)
    args["frontends"] = (
        [unmarshal_Frontend(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListFrontendsResponse(**args)


def unmarshal_ListIpsResponse(data: Any) -> ListIpsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListIpsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("ips", None)
    args["ips"] = [unmarshal_Ip(v) for v in field] if field is not None else None

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListIpsResponse(**args)


def unmarshal_ListLbPrivateNetworksResponse(data: Any) -> ListLbPrivateNetworksResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListLbPrivateNetworksResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("private_network", None)
    args["private_network"] = (
        [unmarshal_PrivateNetwork(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListLbPrivateNetworksResponse(**args)


def unmarshal_ListLbTypesResponse(data: Any) -> ListLbTypesResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListLbTypesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("lb_types", None)
    args["lb_types"] = (
        [unmarshal_LbType(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListLbTypesResponse(**args)


def unmarshal_ListLbsResponse(data: Any) -> ListLbsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListLbsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("lbs", None)
    args["lbs"] = [unmarshal_Lb(v) for v in field] if field is not None else None

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListLbsResponse(**args)


def unmarshal_ListRoutesResponse(data: Any) -> ListRoutesResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListRoutesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("routes", None)
    args["routes"] = [unmarshal_Route(v) for v in field] if field is not None else None

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListRoutesResponse(**args)


def unmarshal_ListSubscriberResponse(data: Any) -> ListSubscriberResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListSubscriberResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("subscribers", None)
    args["subscribers"] = (
        [unmarshal_Subscriber(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListSubscriberResponse(**args)


def unmarshal_SetAclsResponse(data: Any) -> SetAclsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'SetAclsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("acls", None)
    args["acls"] = [unmarshal_Acl(v) for v in field] if field is not None else None

    field = data.get("total_count", None)
    args["total_count"] = field

    return SetAclsResponse(**args)


def marshal_AclActionRedirect(
    request: AclActionRedirect,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.code is not None:
        output["code"] = request.code

    if request.target is not None:
        output["target"] = request.target

    if request.type_ is not None:
        output["type"] = AclActionRedirectRedirectType(request.type_)

    return output


def marshal_AclAction(
    request: AclAction,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.redirect is not None:
        output["redirect"] = marshal_AclActionRedirect(request.redirect, defaults)

    if request.type_ is not None:
        output["type"] = AclActionType(request.type_)

    return output


def marshal_AclMatch(
    request: AclMatch,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.http_filter is not None:
        output["http_filter"] = AclHttpFilter(request.http_filter)

    if request.http_filter_option is not None:
        output["http_filter_option"] = request.http_filter_option

    if request.http_filter_value is not None:
        output["http_filter_value"] = request.http_filter_value

    if request.invert is not None:
        output["invert"] = request.invert

    if request.ip_subnet is not None:
        output["ip_subnet"] = request.ip_subnet

    return output


def marshal_HealthCheckHttpConfig(
    request: HealthCheckHttpConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.code is not None:
        output["code"] = request.code

    if request.host_header is not None:
        output["host_header"] = request.host_header

    if request.method is not None:
        output["method"] = request.method

    if request.uri is not None:
        output["uri"] = request.uri

    return output


def marshal_HealthCheckHttpsConfig(
    request: HealthCheckHttpsConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.code is not None:
        output["code"] = request.code

    if request.host_header is not None:
        output["host_header"] = request.host_header

    if request.method is not None:
        output["method"] = request.method

    if request.sni is not None:
        output["sni"] = request.sni

    if request.uri is not None:
        output["uri"] = request.uri

    return output


def marshal_HealthCheckLdapConfig(
    request: HealthCheckLdapConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    return output


def marshal_HealthCheckMysqlConfig(
    request: HealthCheckMysqlConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.user is not None:
        output["user"] = request.user

    return output


def marshal_HealthCheckPgsqlConfig(
    request: HealthCheckPgsqlConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.user is not None:
        output["user"] = request.user

    return output


def marshal_HealthCheckRedisConfig(
    request: HealthCheckRedisConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    return output


def marshal_HealthCheckTcpConfig(
    request: HealthCheckTcpConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    return output


def marshal_AclSpec(
    request: AclSpec,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.action is not None:
        output["action"] = marshal_AclAction(request.action, defaults)

    if request.description is not None:
        output["description"] = request.description

    if request.index is not None:
        output["index"] = request.index

    if request.match is not None:
        output["match"] = marshal_AclMatch(request.match, defaults)

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_CreateCertificateRequestCustomCertificate(
    request: CreateCertificateRequestCustomCertificate,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.certificate_chain is not None:
        output["certificate_chain"] = request.certificate_chain

    return output


def marshal_CreateCertificateRequestLetsencryptConfig(
    request: CreateCertificateRequestLetsencryptConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.common_name is not None:
        output["common_name"] = request.common_name

    if request.subject_alternative_name is not None:
        output["subject_alternative_name"] = request.subject_alternative_name

    return output


def marshal_HealthCheck(
    request: HealthCheck,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "tcp_config",
                    marshal_HealthCheckTcpConfig(request.tcp_config, defaults)
                    if request.tcp_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "mysql_config",
                    marshal_HealthCheckMysqlConfig(request.mysql_config, defaults)
                    if request.mysql_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "pgsql_config",
                    marshal_HealthCheckPgsqlConfig(request.pgsql_config, defaults)
                    if request.pgsql_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "ldap_config",
                    marshal_HealthCheckLdapConfig(request.ldap_config, defaults)
                    if request.ldap_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "redis_config",
                    marshal_HealthCheckRedisConfig(request.redis_config, defaults)
                    if request.redis_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "http_config",
                    marshal_HealthCheckHttpConfig(request.http_config, defaults)
                    if request.http_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "https_config",
                    marshal_HealthCheckHttpsConfig(request.https_config, defaults)
                    if request.https_config is not None
                    else None,
                ),
            ]
        ),
    )

    if request.check_delay is not None:
        output["check_delay"] = request.check_delay

    if request.check_max_retries is not None:
        output["check_max_retries"] = request.check_max_retries

    if request.check_send_proxy is not None:
        output["check_send_proxy"] = request.check_send_proxy

    if request.check_timeout is not None:
        output["check_timeout"] = request.check_timeout

    if request.port is not None:
        output["port"] = request.port

    if request.transient_check_delay is not None:
        output["transient_check_delay"] = request.transient_check_delay

    return output


def marshal_PrivateNetworkDHCPConfig(
    request: PrivateNetworkDHCPConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.ip_id is not None:
        output["ip_id"] = request.ip_id

    return output


def marshal_PrivateNetworkIpamConfig(
    request: PrivateNetworkIpamConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    return output


def marshal_PrivateNetworkStaticConfig(
    request: PrivateNetworkStaticConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.ip_address is not None:
        output["ip_address"] = request.ip_address

    return output


def marshal_RouteMatch(
    request: RouteMatch,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "sni", request.sni if request.sni is not None else None
                ),
                OneOfPossibility(
                    "host_header",
                    request.host_header if request.host_header is not None else None,
                ),
            ]
        ),
    )

    return output


def marshal_SubscriberEmailConfig(
    request: SubscriberEmailConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.email is not None:
        output["email"] = request.email

    return output


def marshal_SubscriberWebhookConfig(
    request: SubscriberWebhookConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.uri is not None:
        output["uri"] = request.uri

    return output


def marshal_AddBackendServersRequest(
    request: AddBackendServersRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.server_ip is not None:
        output["server_ip"] = request.server_ip

    return output


def marshal_AttachPrivateNetworkRequest(
    request: AttachPrivateNetworkRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "static_config",
                    marshal_PrivateNetworkStaticConfig(request.static_config, defaults)
                    if request.static_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "dhcp_config",
                    marshal_PrivateNetworkDHCPConfig(request.dhcp_config, defaults)
                    if request.dhcp_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "ipam_config",
                    marshal_PrivateNetworkIpamConfig(request.ipam_config, defaults)
                    if request.ipam_config is not None
                    else None,
                ),
            ]
        ),
    )

    return output


def marshal_CreateAclRequest(
    request: CreateAclRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.action is not None:
        output["action"] = marshal_AclAction(request.action, defaults)

    if request.description is not None:
        output["description"] = request.description

    if request.index is not None:
        output["index"] = request.index

    if request.match is not None:
        output["match"] = marshal_AclMatch(request.match, defaults)

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_CreateBackendRequest(
    request: CreateBackendRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.failover_host is not None:
        output["failover_host"] = request.failover_host

    if request.forward_port is not None:
        output["forward_port"] = request.forward_port

    if request.forward_port_algorithm is not None:
        output["forward_port_algorithm"] = ForwardPortAlgorithm(
            request.forward_port_algorithm
        )

    if request.forward_protocol is not None:
        output["forward_protocol"] = Protocol(request.forward_protocol)

    if request.health_check is not None:
        output["health_check"] = marshal_HealthCheck(request.health_check, defaults)

    if request.ignore_ssl_server_verify is not None:
        output["ignore_ssl_server_verify"] = request.ignore_ssl_server_verify

    if request.max_connections is not None:
        output["max_connections"] = request.max_connections

    if request.max_retries is not None:
        output["max_retries"] = request.max_retries

    if request.name is not None:
        output["name"] = request.name

    if request.on_marked_down_action is not None:
        output["on_marked_down_action"] = OnMarkedDownAction(
            request.on_marked_down_action
        )

    if request.proxy_protocol is not None:
        output["proxy_protocol"] = ProxyProtocol(request.proxy_protocol)

    if request.redispatch_attempt_count is not None:
        output["redispatch_attempt_count"] = request.redispatch_attempt_count

    if request.send_proxy_v2 is not None:
        output["send_proxy_v2"] = request.send_proxy_v2

    if request.server_ip is not None:
        output["server_ip"] = request.server_ip

    if request.ssl_bridging is not None:
        output["ssl_bridging"] = request.ssl_bridging

    if request.sticky_sessions is not None:
        output["sticky_sessions"] = StickySessionsType(request.sticky_sessions)

    if request.sticky_sessions_cookie_name is not None:
        output["sticky_sessions_cookie_name"] = request.sticky_sessions_cookie_name

    if request.timeout_connect is not None:
        output["timeout_connect"] = request.timeout_connect

    if request.timeout_queue is not None:
        output["timeout_queue"] = request.timeout_queue

    if request.timeout_server is not None:
        output["timeout_server"] = request.timeout_server

    if request.timeout_tunnel is not None:
        output["timeout_tunnel"] = request.timeout_tunnel

    return output


def marshal_CreateCertificateRequest(
    request: CreateCertificateRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "letsencrypt",
                    marshal_CreateCertificateRequestLetsencryptConfig(
                        request.letsencrypt, defaults
                    )
                    if request.letsencrypt is not None
                    else None,
                ),
                OneOfPossibility(
                    "custom_certificate",
                    marshal_CreateCertificateRequestCustomCertificate(
                        request.custom_certificate, defaults
                    )
                    if request.custom_certificate is not None
                    else None,
                ),
            ]
        ),
    )

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_CreateFrontendRequest(
    request: CreateFrontendRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.backend_id is not None:
        output["backend_id"] = request.backend_id

    if request.certificate_id is not None:
        output["certificate_id"] = request.certificate_id

    if request.certificate_ids is not None:
        output["certificate_ids"] = request.certificate_ids

    if request.enable_http3 is not None:
        output["enable_http3"] = request.enable_http3

    if request.inbound_port is not None:
        output["inbound_port"] = request.inbound_port

    if request.name is not None:
        output["name"] = request.name

    if request.timeout_client is not None:
        output["timeout_client"] = request.timeout_client

    return output


def marshal_CreateIpRequest(
    request: CreateIpRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "project_id",
                    request.project_id or defaults.default_project_id
                    if request.project_id is not None
                    else None,
                    defaults.default_project_id,
                ),
                OneOfPossibility(
                    "organization_id",
                    request.organization_id or defaults.default_organization_id
                    if request.organization_id is not None
                    else None,
                    defaults.default_organization_id,
                ),
            ]
        ),
    )

    if request.reverse is not None:
        output["reverse"] = request.reverse

    return output


def marshal_CreateLbRequest(
    request: CreateLbRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "project_id",
                    request.project_id or defaults.default_project_id
                    if request.project_id is not None
                    else None,
                    defaults.default_project_id,
                ),
                OneOfPossibility(
                    "organization_id",
                    request.organization_id or defaults.default_organization_id
                    if request.organization_id is not None
                    else None,
                    defaults.default_organization_id,
                ),
            ]
        ),
    )

    if request.assign_flexible_ip is not None:
        output["assign_flexible_ip"] = request.assign_flexible_ip

    if request.description is not None:
        output["description"] = request.description

    if request.ip_id is not None:
        output["ip_id"] = request.ip_id

    if request.name is not None:
        output["name"] = request.name

    if request.ssl_compatibility_level is not None:
        output["ssl_compatibility_level"] = SSLCompatibilityLevel(
            request.ssl_compatibility_level
        )

    if request.tags is not None:
        output["tags"] = request.tags

    if request.type_ is not None:
        output["type"] = request.type_

    return output


def marshal_CreateRouteRequest(
    request: CreateRouteRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.backend_id is not None:
        output["backend_id"] = request.backend_id

    if request.frontend_id is not None:
        output["frontend_id"] = request.frontend_id

    if request.match is not None:
        output["match"] = marshal_RouteMatch(request.match, defaults)

    return output


def marshal_CreateSubscriberRequest(
    request: CreateSubscriberRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "project_id",
                    request.project_id or defaults.default_project_id
                    if request.project_id is not None
                    else None,
                    defaults.default_project_id,
                ),
                OneOfPossibility(
                    "organization_id",
                    request.organization_id or defaults.default_organization_id
                    if request.organization_id is not None
                    else None,
                    defaults.default_organization_id,
                ),
            ]
        ),
    )
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "email_config",
                    marshal_SubscriberEmailConfig(request.email_config, defaults)
                    if request.email_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "webhook_config",
                    marshal_SubscriberWebhookConfig(request.webhook_config, defaults)
                    if request.webhook_config is not None
                    else None,
                ),
            ]
        ),
    )

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_MigrateLbRequest(
    request: MigrateLbRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.type_ is not None:
        output["type"] = request.type_

    return output


def marshal_RemoveBackendServersRequest(
    request: RemoveBackendServersRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.server_ip is not None:
        output["server_ip"] = request.server_ip

    return output


def marshal_SetBackendServersRequest(
    request: SetBackendServersRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.server_ip is not None:
        output["server_ip"] = request.server_ip

    return output


def marshal_SubscribeToLbRequest(
    request: SubscribeToLbRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.subscriber_id is not None:
        output["subscriber_id"] = request.subscriber_id

    return output


def marshal_UpdateAclRequest(
    request: UpdateAclRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.action is not None:
        output["action"] = marshal_AclAction(request.action, defaults)

    if request.description is not None:
        output["description"] = request.description

    if request.index is not None:
        output["index"] = request.index

    if request.match is not None:
        output["match"] = marshal_AclMatch(request.match, defaults)

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_UpdateBackendRequest(
    request: UpdateBackendRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.failover_host is not None:
        output["failover_host"] = request.failover_host

    if request.forward_port is not None:
        output["forward_port"] = request.forward_port

    if request.forward_port_algorithm is not None:
        output["forward_port_algorithm"] = ForwardPortAlgorithm(
            request.forward_port_algorithm
        )

    if request.forward_protocol is not None:
        output["forward_protocol"] = Protocol(request.forward_protocol)

    if request.ignore_ssl_server_verify is not None:
        output["ignore_ssl_server_verify"] = request.ignore_ssl_server_verify

    if request.max_connections is not None:
        output["max_connections"] = request.max_connections

    if request.max_retries is not None:
        output["max_retries"] = request.max_retries

    if request.name is not None:
        output["name"] = request.name

    if request.on_marked_down_action is not None:
        output["on_marked_down_action"] = OnMarkedDownAction(
            request.on_marked_down_action
        )

    if request.proxy_protocol is not None:
        output["proxy_protocol"] = ProxyProtocol(request.proxy_protocol)

    if request.redispatch_attempt_count is not None:
        output["redispatch_attempt_count"] = request.redispatch_attempt_count

    if request.send_proxy_v2 is not None:
        output["send_proxy_v2"] = request.send_proxy_v2

    if request.ssl_bridging is not None:
        output["ssl_bridging"] = request.ssl_bridging

    if request.sticky_sessions is not None:
        output["sticky_sessions"] = StickySessionsType(request.sticky_sessions)

    if request.sticky_sessions_cookie_name is not None:
        output["sticky_sessions_cookie_name"] = request.sticky_sessions_cookie_name

    if request.timeout_connect is not None:
        output["timeout_connect"] = request.timeout_connect

    if request.timeout_queue is not None:
        output["timeout_queue"] = request.timeout_queue

    if request.timeout_server is not None:
        output["timeout_server"] = request.timeout_server

    if request.timeout_tunnel is not None:
        output["timeout_tunnel"] = request.timeout_tunnel

    return output


def marshal_UpdateCertificateRequest(
    request: UpdateCertificateRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_UpdateFrontendRequest(
    request: UpdateFrontendRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.backend_id is not None:
        output["backend_id"] = request.backend_id

    if request.certificate_id is not None:
        output["certificate_id"] = request.certificate_id

    if request.certificate_ids is not None:
        output["certificate_ids"] = request.certificate_ids

    if request.enable_http3 is not None:
        output["enable_http3"] = request.enable_http3

    if request.inbound_port is not None:
        output["inbound_port"] = request.inbound_port

    if request.name is not None:
        output["name"] = request.name

    if request.timeout_client is not None:
        output["timeout_client"] = request.timeout_client

    return output


def marshal_UpdateHealthCheckRequest(
    request: UpdateHealthCheckRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "tcp_config",
                    marshal_HealthCheckTcpConfig(request.tcp_config, defaults)
                    if request.tcp_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "mysql_config",
                    marshal_HealthCheckMysqlConfig(request.mysql_config, defaults)
                    if request.mysql_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "pgsql_config",
                    marshal_HealthCheckPgsqlConfig(request.pgsql_config, defaults)
                    if request.pgsql_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "ldap_config",
                    marshal_HealthCheckLdapConfig(request.ldap_config, defaults)
                    if request.ldap_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "redis_config",
                    marshal_HealthCheckRedisConfig(request.redis_config, defaults)
                    if request.redis_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "http_config",
                    marshal_HealthCheckHttpConfig(request.http_config, defaults)
                    if request.http_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "https_config",
                    marshal_HealthCheckHttpsConfig(request.https_config, defaults)
                    if request.https_config is not None
                    else None,
                ),
            ]
        ),
    )

    if request.check_delay is not None:
        output["check_delay"] = request.check_delay

    if request.check_max_retries is not None:
        output["check_max_retries"] = request.check_max_retries

    if request.check_send_proxy is not None:
        output["check_send_proxy"] = request.check_send_proxy

    if request.check_timeout is not None:
        output["check_timeout"] = request.check_timeout

    if request.port is not None:
        output["port"] = request.port

    if request.transient_check_delay is not None:
        output["transient_check_delay"] = request.transient_check_delay

    return output


def marshal_UpdateIpRequest(
    request: UpdateIpRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.reverse is not None:
        output["reverse"] = request.reverse

    return output


def marshal_UpdateLbRequest(
    request: UpdateLbRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.description is not None:
        output["description"] = request.description

    if request.name is not None:
        output["name"] = request.name

    if request.ssl_compatibility_level is not None:
        output["ssl_compatibility_level"] = SSLCompatibilityLevel(
            request.ssl_compatibility_level
        )

    if request.tags is not None:
        output["tags"] = request.tags

    return output


def marshal_UpdateRouteRequest(
    request: UpdateRouteRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.backend_id is not None:
        output["backend_id"] = request.backend_id

    if request.match is not None:
        output["match"] = marshal_RouteMatch(request.match, defaults)

    return output


def marshal_UpdateSubscriberRequest(
    request: UpdateSubscriberRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "email_config",
                    marshal_SubscriberEmailConfig(request.email_config, defaults)
                    if request.email_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "webhook_config",
                    marshal_SubscriberWebhookConfig(request.webhook_config, defaults)
                    if request.webhook_config is not None
                    else None,
                ),
            ]
        ),
    )

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_ZonedApiAddBackendServersRequest(
    request: ZonedApiAddBackendServersRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.server_ip is not None:
        output["server_ip"] = request.server_ip

    return output


def marshal_ZonedApiAttachPrivateNetworkRequest(
    request: ZonedApiAttachPrivateNetworkRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "static_config",
                    marshal_PrivateNetworkStaticConfig(request.static_config, defaults)
                    if request.static_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "dhcp_config",
                    marshal_PrivateNetworkDHCPConfig(request.dhcp_config, defaults)
                    if request.dhcp_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "ipam_config",
                    marshal_PrivateNetworkIpamConfig(request.ipam_config, defaults)
                    if request.ipam_config is not None
                    else None,
                ),
            ]
        ),
    )

    return output


def marshal_ZonedApiCreateAclRequest(
    request: ZonedApiCreateAclRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.action is not None:
        output["action"] = marshal_AclAction(request.action, defaults)

    if request.description is not None:
        output["description"] = request.description

    if request.index is not None:
        output["index"] = request.index

    if request.match is not None:
        output["match"] = marshal_AclMatch(request.match, defaults)

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_ZonedApiCreateBackendRequest(
    request: ZonedApiCreateBackendRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.failover_host is not None:
        output["failover_host"] = request.failover_host

    if request.forward_port is not None:
        output["forward_port"] = request.forward_port

    if request.forward_port_algorithm is not None:
        output["forward_port_algorithm"] = ForwardPortAlgorithm(
            request.forward_port_algorithm
        )

    if request.forward_protocol is not None:
        output["forward_protocol"] = Protocol(request.forward_protocol)

    if request.health_check is not None:
        output["health_check"] = marshal_HealthCheck(request.health_check, defaults)

    if request.ignore_ssl_server_verify is not None:
        output["ignore_ssl_server_verify"] = request.ignore_ssl_server_verify

    if request.max_connections is not None:
        output["max_connections"] = request.max_connections

    if request.max_retries is not None:
        output["max_retries"] = request.max_retries

    if request.name is not None:
        output["name"] = request.name

    if request.on_marked_down_action is not None:
        output["on_marked_down_action"] = OnMarkedDownAction(
            request.on_marked_down_action
        )

    if request.proxy_protocol is not None:
        output["proxy_protocol"] = ProxyProtocol(request.proxy_protocol)

    if request.redispatch_attempt_count is not None:
        output["redispatch_attempt_count"] = request.redispatch_attempt_count

    if request.send_proxy_v2 is not None:
        output["send_proxy_v2"] = request.send_proxy_v2

    if request.server_ip is not None:
        output["server_ip"] = request.server_ip

    if request.ssl_bridging is not None:
        output["ssl_bridging"] = request.ssl_bridging

    if request.sticky_sessions is not None:
        output["sticky_sessions"] = StickySessionsType(request.sticky_sessions)

    if request.sticky_sessions_cookie_name is not None:
        output["sticky_sessions_cookie_name"] = request.sticky_sessions_cookie_name

    if request.timeout_connect is not None:
        output["timeout_connect"] = request.timeout_connect

    if request.timeout_queue is not None:
        output["timeout_queue"] = request.timeout_queue

    if request.timeout_server is not None:
        output["timeout_server"] = request.timeout_server

    if request.timeout_tunnel is not None:
        output["timeout_tunnel"] = request.timeout_tunnel

    return output


def marshal_ZonedApiCreateCertificateRequest(
    request: ZonedApiCreateCertificateRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "letsencrypt",
                    marshal_CreateCertificateRequestLetsencryptConfig(
                        request.letsencrypt, defaults
                    )
                    if request.letsencrypt is not None
                    else None,
                ),
                OneOfPossibility(
                    "custom_certificate",
                    marshal_CreateCertificateRequestCustomCertificate(
                        request.custom_certificate, defaults
                    )
                    if request.custom_certificate is not None
                    else None,
                ),
            ]
        ),
    )

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_ZonedApiCreateFrontendRequest(
    request: ZonedApiCreateFrontendRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.backend_id is not None:
        output["backend_id"] = request.backend_id

    if request.certificate_id is not None:
        output["certificate_id"] = request.certificate_id

    if request.certificate_ids is not None:
        output["certificate_ids"] = request.certificate_ids

    if request.enable_http3 is not None:
        output["enable_http3"] = request.enable_http3

    if request.inbound_port is not None:
        output["inbound_port"] = request.inbound_port

    if request.name is not None:
        output["name"] = request.name

    if request.timeout_client is not None:
        output["timeout_client"] = request.timeout_client

    return output


def marshal_ZonedApiCreateIpRequest(
    request: ZonedApiCreateIpRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "project_id",
                    request.project_id or defaults.default_project_id
                    if request.project_id is not None
                    else None,
                    defaults.default_project_id,
                ),
                OneOfPossibility(
                    "organization_id",
                    request.organization_id or defaults.default_organization_id
                    if request.organization_id is not None
                    else None,
                    defaults.default_organization_id,
                ),
            ]
        ),
    )

    if request.reverse is not None:
        output["reverse"] = request.reverse

    return output


def marshal_ZonedApiCreateLbRequest(
    request: ZonedApiCreateLbRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "project_id",
                    request.project_id or defaults.default_project_id
                    if request.project_id is not None
                    else None,
                    defaults.default_project_id,
                ),
                OneOfPossibility(
                    "organization_id",
                    request.organization_id or defaults.default_organization_id
                    if request.organization_id is not None
                    else None,
                    defaults.default_organization_id,
                ),
            ]
        ),
    )

    if request.assign_flexible_ip is not None:
        output["assign_flexible_ip"] = request.assign_flexible_ip

    if request.description is not None:
        output["description"] = request.description

    if request.ip_id is not None:
        output["ip_id"] = request.ip_id

    if request.name is not None:
        output["name"] = request.name

    if request.ssl_compatibility_level is not None:
        output["ssl_compatibility_level"] = SSLCompatibilityLevel(
            request.ssl_compatibility_level
        )

    if request.tags is not None:
        output["tags"] = request.tags

    if request.type_ is not None:
        output["type"] = request.type_

    return output


def marshal_ZonedApiCreateRouteRequest(
    request: ZonedApiCreateRouteRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.backend_id is not None:
        output["backend_id"] = request.backend_id

    if request.frontend_id is not None:
        output["frontend_id"] = request.frontend_id

    if request.match is not None:
        output["match"] = marshal_RouteMatch(request.match, defaults)

    return output


def marshal_ZonedApiCreateSubscriberRequest(
    request: ZonedApiCreateSubscriberRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "project_id",
                    request.project_id or defaults.default_project_id
                    if request.project_id is not None
                    else None,
                    defaults.default_project_id,
                ),
                OneOfPossibility(
                    "organization_id",
                    request.organization_id or defaults.default_organization_id
                    if request.organization_id is not None
                    else None,
                    defaults.default_organization_id,
                ),
            ]
        ),
    )
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "email_config",
                    marshal_SubscriberEmailConfig(request.email_config, defaults)
                    if request.email_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "webhook_config",
                    marshal_SubscriberWebhookConfig(request.webhook_config, defaults)
                    if request.webhook_config is not None
                    else None,
                ),
            ]
        ),
    )

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_ZonedApiMigrateLbRequest(
    request: ZonedApiMigrateLbRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.type_ is not None:
        output["type"] = request.type_

    return output


def marshal_ZonedApiRemoveBackendServersRequest(
    request: ZonedApiRemoveBackendServersRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.server_ip is not None:
        output["server_ip"] = request.server_ip

    return output


def marshal_ZonedApiSetAclsRequest(
    request: ZonedApiSetAclsRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.acls is not None:
        output["acls"] = [marshal_AclSpec(v, defaults) for v in request.acls]

    return output


def marshal_ZonedApiSetBackendServersRequest(
    request: ZonedApiSetBackendServersRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.server_ip is not None:
        output["server_ip"] = request.server_ip

    return output


def marshal_ZonedApiSubscribeToLbRequest(
    request: ZonedApiSubscribeToLbRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.subscriber_id is not None:
        output["subscriber_id"] = request.subscriber_id

    return output


def marshal_ZonedApiUpdateAclRequest(
    request: ZonedApiUpdateAclRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.action is not None:
        output["action"] = marshal_AclAction(request.action, defaults)

    if request.description is not None:
        output["description"] = request.description

    if request.index is not None:
        output["index"] = request.index

    if request.match is not None:
        output["match"] = marshal_AclMatch(request.match, defaults)

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_ZonedApiUpdateBackendRequest(
    request: ZonedApiUpdateBackendRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.failover_host is not None:
        output["failover_host"] = request.failover_host

    if request.forward_port is not None:
        output["forward_port"] = request.forward_port

    if request.forward_port_algorithm is not None:
        output["forward_port_algorithm"] = ForwardPortAlgorithm(
            request.forward_port_algorithm
        )

    if request.forward_protocol is not None:
        output["forward_protocol"] = Protocol(request.forward_protocol)

    if request.ignore_ssl_server_verify is not None:
        output["ignore_ssl_server_verify"] = request.ignore_ssl_server_verify

    if request.max_connections is not None:
        output["max_connections"] = request.max_connections

    if request.max_retries is not None:
        output["max_retries"] = request.max_retries

    if request.name is not None:
        output["name"] = request.name

    if request.on_marked_down_action is not None:
        output["on_marked_down_action"] = OnMarkedDownAction(
            request.on_marked_down_action
        )

    if request.proxy_protocol is not None:
        output["proxy_protocol"] = ProxyProtocol(request.proxy_protocol)

    if request.redispatch_attempt_count is not None:
        output["redispatch_attempt_count"] = request.redispatch_attempt_count

    if request.send_proxy_v2 is not None:
        output["send_proxy_v2"] = request.send_proxy_v2

    if request.ssl_bridging is not None:
        output["ssl_bridging"] = request.ssl_bridging

    if request.sticky_sessions is not None:
        output["sticky_sessions"] = StickySessionsType(request.sticky_sessions)

    if request.sticky_sessions_cookie_name is not None:
        output["sticky_sessions_cookie_name"] = request.sticky_sessions_cookie_name

    if request.timeout_connect is not None:
        output["timeout_connect"] = request.timeout_connect

    if request.timeout_queue is not None:
        output["timeout_queue"] = request.timeout_queue

    if request.timeout_server is not None:
        output["timeout_server"] = request.timeout_server

    if request.timeout_tunnel is not None:
        output["timeout_tunnel"] = request.timeout_tunnel

    return output


def marshal_ZonedApiUpdateCertificateRequest(
    request: ZonedApiUpdateCertificateRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_ZonedApiUpdateFrontendRequest(
    request: ZonedApiUpdateFrontendRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.backend_id is not None:
        output["backend_id"] = request.backend_id

    if request.certificate_id is not None:
        output["certificate_id"] = request.certificate_id

    if request.certificate_ids is not None:
        output["certificate_ids"] = request.certificate_ids

    if request.enable_http3 is not None:
        output["enable_http3"] = request.enable_http3

    if request.inbound_port is not None:
        output["inbound_port"] = request.inbound_port

    if request.name is not None:
        output["name"] = request.name

    if request.timeout_client is not None:
        output["timeout_client"] = request.timeout_client

    return output


def marshal_ZonedApiUpdateHealthCheckRequest(
    request: ZonedApiUpdateHealthCheckRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "tcp_config",
                    marshal_HealthCheckTcpConfig(request.tcp_config, defaults)
                    if request.tcp_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "mysql_config",
                    marshal_HealthCheckMysqlConfig(request.mysql_config, defaults)
                    if request.mysql_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "pgsql_config",
                    marshal_HealthCheckPgsqlConfig(request.pgsql_config, defaults)
                    if request.pgsql_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "ldap_config",
                    marshal_HealthCheckLdapConfig(request.ldap_config, defaults)
                    if request.ldap_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "redis_config",
                    marshal_HealthCheckRedisConfig(request.redis_config, defaults)
                    if request.redis_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "http_config",
                    marshal_HealthCheckHttpConfig(request.http_config, defaults)
                    if request.http_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "https_config",
                    marshal_HealthCheckHttpsConfig(request.https_config, defaults)
                    if request.https_config is not None
                    else None,
                ),
            ]
        ),
    )

    if request.check_delay is not None:
        output["check_delay"] = request.check_delay

    if request.check_max_retries is not None:
        output["check_max_retries"] = request.check_max_retries

    if request.check_send_proxy is not None:
        output["check_send_proxy"] = request.check_send_proxy

    if request.check_timeout is not None:
        output["check_timeout"] = request.check_timeout

    if request.port is not None:
        output["port"] = request.port

    if request.transient_check_delay is not None:
        output["transient_check_delay"] = request.transient_check_delay

    return output


def marshal_ZonedApiUpdateIpRequest(
    request: ZonedApiUpdateIpRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.reverse is not None:
        output["reverse"] = request.reverse

    return output


def marshal_ZonedApiUpdateLbRequest(
    request: ZonedApiUpdateLbRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.description is not None:
        output["description"] = request.description

    if request.name is not None:
        output["name"] = request.name

    if request.ssl_compatibility_level is not None:
        output["ssl_compatibility_level"] = SSLCompatibilityLevel(
            request.ssl_compatibility_level
        )

    if request.tags is not None:
        output["tags"] = request.tags

    return output


def marshal_ZonedApiUpdateRouteRequest(
    request: ZonedApiUpdateRouteRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.backend_id is not None:
        output["backend_id"] = request.backend_id

    if request.match is not None:
        output["match"] = marshal_RouteMatch(request.match, defaults)

    return output


def marshal_ZonedApiUpdateSubscriberRequest(
    request: ZonedApiUpdateSubscriberRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility(
                    "email_config",
                    marshal_SubscriberEmailConfig(request.email_config, defaults)
                    if request.email_config is not None
                    else None,
                ),
                OneOfPossibility(
                    "webhook_config",
                    marshal_SubscriberWebhookConfig(request.webhook_config, defaults)
                    if request.webhook_config is not None
                    else None,
                ),
            ]
        ),
    )

    if request.name is not None:
        output["name"] = request.name

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional


class BearerType(str, Enum):
    UNKNOWN_BEARER_TYPE = "unknown_bearer_type"
    USER = "user"
    APPLICATION = "application"

    def __str__(self) -> str:
        return str(self.value)


class ListAPIKeysRequestOrderBy(str, Enum):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    UPDATED_AT_ASC = "updated_at_asc"
    UPDATED_AT_DESC = "updated_at_desc"
    EXPIRES_AT_ASC = "expires_at_asc"
    EXPIRES_AT_DESC = "expires_at_desc"
    ACCESS_KEY_ASC = "access_key_asc"
    ACCESS_KEY_DESC = "access_key_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListApplicationsRequestOrderBy(str, Enum):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    UPDATED_AT_ASC = "updated_at_asc"
    UPDATED_AT_DESC = "updated_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListGroupsRequestOrderBy(str, Enum):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    UPDATED_AT_ASC = "updated_at_asc"
    UPDATED_AT_DESC = "updated_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListJWTsRequestOrderBy(str, Enum):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    UPDATED_AT_ASC = "updated_at_asc"
    UPDATED_AT_DESC = "updated_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListPermissionSetsRequestOrderBy(str, Enum):
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListPoliciesRequestOrderBy(str, Enum):
    POLICY_NAME_ASC = "policy_name_asc"
    POLICY_NAME_DESC = "policy_name_desc"
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListQuotaRequestOrderBy(str, Enum):
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListSSHKeysRequestOrderBy(str, Enum):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    UPDATED_AT_ASC = "updated_at_asc"
    UPDATED_AT_DESC = "updated_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListUsersRequestOrderBy(str, Enum):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    UPDATED_AT_ASC = "updated_at_asc"
    UPDATED_AT_DESC = "updated_at_desc"
    EMAIL_ASC = "email_asc"
    EMAIL_DESC = "email_desc"
    LAST_LOGIN_ASC = "last_login_asc"
    LAST_LOGIN_DESC = "last_login_desc"

    def __str__(self) -> str:
        return str(self.value)


class PermissionSetScopeType(str, Enum):
    UNKNOWN_SCOPE_TYPE = "unknown_scope_type"
    PROJECTS = "projects"
    ORGANIZATION = "organization"
    ACCOUNT_ROOT_USER = "account_root_user"

    def __str__(self) -> str:
        return str(self.value)


class UserStatus(str, Enum):
    UNKNOWN_STATUS = "unknown_status"
    INVITATION_PENDING = "invitation_pending"
    ACTIVATED = "activated"

    def __str__(self) -> str:
        return str(self.value)


class UserType(str, Enum):
    UNKNOWN_TYPE = "unknown_type"
    GUEST = "guest"
    OWNER = "owner"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class APIKey:
    """
    Api key.
    """

    access_key: str
    """
    Access key of the API key.
    """

    secret_key: Optional[str]
    """
    Secret key of the API Key.
    """

    application_id: Optional[str]
    """
    ID of application that bears the API key.
    
    One-of ('bearer'): at most one of 'application_id', 'user_id' could be set.
    """

    user_id: Optional[str]
    """
    ID of user that bears the API key.
    
    One-of ('bearer'): at most one of 'application_id', 'user_id' could be set.
    """

    description: str
    """
    Description of API key.
    """

    created_at: Optional[datetime]
    """
    Date and time of API key creation.
    """

    updated_at: Optional[datetime]
    """
    Date and time of last API key update.
    """

    expires_at: Optional[datetime]
    """
    Date and time of API key expiration.
    """

    default_project_id: str
    """
    Default Project ID specified for this API key.
    """

    editable: bool
    """
    Defines whether or not the API key is editable.
    """

    creation_ip: str
    """
    IP address of the device that created the API key.
    """


@dataclass
class Application:
    """
    Application.
    """

    id: str
    """
    ID of the application.
    """

    name: str
    """
    Name of the application.
    """

    description: str
    """
    Description of the application.
    """

    created_at: Optional[datetime]
    """
    Date and time application was created.
    """

    updated_at: Optional[datetime]
    """
    Date and time of last application update.
    """

    organization_id: str
    """
    ID of the Organization.
    """

    editable: bool
    """
    Defines whether or not the application is editable.
    """

    nb_api_keys: int
    """
    Number of API keys attributed to the application.
    """


@dataclass
class Group:
    """
    Group.
    """

    id: str
    """
    ID of the group.
    """

    created_at: Optional[datetime]
    """
    Date and time of group creation.
    """

    updated_at: Optional[datetime]
    """
    Date and time of last group update.
    """

    organization_id: str
    """
    ID of Organization linked to the group.
    """

    name: str
    """
    Name of the group.
    """

    description: str
    """
    Description of the group.
    """

    user_ids: List[str]
    """
    IDs of users attached to this group.
    """

    application_ids: List[str]
    """
    IDs of applications attached to this group.
    """


@dataclass
class JWT:
    """
    Jwt.
    """

    jti: str
    """
    JWT ID.
    """

    issuer_id: str
    """
    ID of the user who issued the JWT.
    """

    audience_id: str
    """
    ID of the user targeted by the JWT.
    """

    created_at: Optional[datetime]
    """
    Creation date of the JWT.
    """

    updated_at: Optional[datetime]
    """
    Last update date of the JWT.
    """

    expires_at: Optional[datetime]
    """
    Expiration date of the JWT.
    """

    ip: str
    """
    IP address used during the creation of the JWT.
    """

    user_agent: str
    """
    User-agent used during the creation of the JWT.
    """


@dataclass
class ListAPIKeysResponse:
    """
    List api keys response.
    """

    api_keys: List[APIKey]
    """
    List of API keys.
    """

    total_count: int
    """
    Total count of API Keys.
    """


@dataclass
class ListApplicationsResponse:
    """
    List applications response.
    """

    applications: List[Application]
    """
    List of applications.
    """

    total_count: int
    """
    Total count of applications.
    """


@dataclass
class ListGroupsResponse:
    """
    List groups response.
    """

    groups: List[Group]
    """
    List of groups.
    """

    total_count: int
    """
    Total count of groups.
    """


@dataclass
class ListJWTsResponse:
    jwts: List[JWT]

    total_count: int


@dataclass
class ListPermissionSetsResponse:
    """
    List permission sets response.
    """

    permission_sets: List[PermissionSet]
    """
    List of permission sets.
    """

    total_count: int
    """
    Total count of permission sets.
    """


@dataclass
class ListPoliciesResponse:
    """
    List policies response.
    """

    policies: List[Policy]
    """
    List of policies.
    """

    total_count: int
    """
    Total count of policies.
    """


@dataclass
class ListQuotaResponse:
    """
    List quota response.
    """

    quota: List[Quotum]
    """
    List of quota.
    """

    total_count: int
    """
    Total count of quota.
    """


@dataclass
class ListRulesResponse:
    """
    List rules response.
    """

    rules: List[Rule]
    """
    Rules of the policy.
    """

    total_count: int
    """
    Total count of rules.
    """


@dataclass
class ListSSHKeysResponse:
    """
    List ssh keys response.
    """

    ssh_keys: List[SSHKey]
    """
    List of SSH keys.
    """

    total_count: int
    """
    Total count of SSH keys.
    """


@dataclass
class ListUsersResponse:
    """
    List users response.
    """

    users: List[User]
    """
    List of users.
    """

    total_count: int
    """
    Total count of users.
    """


@dataclass
class PermissionSet:
    """
    Permission set.
    """

    id: str
    """
    Id of the permission set.
    """

    name: str
    """
    Name of the permission set.
    """

    scope_type: PermissionSetScopeType
    """
    Scope of the permission set.
    """

    description: str
    """
    Description of the permission set.
    """

    categories: Optional[List[str]]
    """
    Categories of the permission set.
    """


@dataclass
class Policy:
    """
    Policy.
    """

    id: str
    """
    Id of the policy.
    """

    name: str
    """
    Name of the policy.
    """

    description: str
    """
    Description of the policy.
    """

    organization_id: str
    """
    Organization ID of the policy.
    """

    created_at: Optional[datetime]
    """
    Date and time of policy creation.
    """

    updated_at: Optional[datetime]
    """
    Date and time of last policy update.
    """

    editable: bool
    """
    Defines whether or not a policy is editable.
    """

    nb_rules: int
    """
    Number of rules of the policy.
    """

    nb_scopes: int
    """
    Number of policy scopes.
    """

    nb_permission_sets: int
    """
    Number of permission sets of the policy.
    """

    user_id: Optional[str]
    """
    ID of the user attributed to the policy.
    
    One-of ('principal'): at most one of 'user_id', 'group_id', 'application_id', 'no_principal' could be set.
    """

    group_id: Optional[str]
    """
    ID of the group attributed to the policy.
    
    One-of ('principal'): at most one of 'user_id', 'group_id', 'application_id', 'no_principal' could be set.
    """

    application_id: Optional[str]
    """
    ID of the application attributed to the policy.
    
    One-of ('principal'): at most one of 'user_id', 'group_id', 'application_id', 'no_principal' could be set.
    """

    no_principal: Optional[bool]
    """
    Defines whether or not a policy is attributed to a principal.
    
    One-of ('principal'): at most one of 'user_id', 'group_id', 'application_id', 'no_principal' could be set.
    """


@dataclass
class Quotum:
    """
    Quotum.
    """

    name: str
    """
    Name of the quota.
    """

    limit: Optional[int]
    """
    Maximum limit of the quota.
    
    One-of ('value'): at most one of 'limit', 'unlimited' could be set.
    """

    unlimited: Optional[bool]
    """
    Defines whether or not the quota is unlimited.
    
    One-of ('value'): at most one of 'limit', 'unlimited' could be set.
    """


@dataclass
class Rule:
    """
    Rule.
    """

    id: str
    """
    Id of rule.
    """

    permission_set_names: Optional[List[str]]
    """
    Names of permission sets bound to the rule.
    """

    permission_sets_scope_type: PermissionSetScopeType
    """
    Permission_set_names have the same scope_type.
    """

    project_ids: Optional[List[str]]
    """
    List of Project IDs the rule is scoped to.
    
    One-of ('scope'): at most one of 'project_ids', 'organization_id', 'account_root_user_id' could be set.
    """

    organization_id: Optional[str]
    """
    ID of Organization the rule is scoped to.
    
    One-of ('scope'): at most one of 'project_ids', 'organization_id', 'account_root_user_id' could be set.
    """

    account_root_user_id: Optional[str]
    """
    ID of account root user the rule is scoped to.
    
    One-of ('scope'): at most one of 'project_ids', 'organization_id', 'account_root_user_id' could be set.
    """


@dataclass
class RuleSpecs:
    """
    Rule specs.
    """

    permission_set_names: Optional[List[str]]
    """
    Names of permission sets bound to the rule.
    """

    project_ids: Optional[List[str]]
    """
    List of Project IDs the rule is scoped to.
    
    One-of ('scope'): at most one of 'project_ids', 'organization_id' could be set.
    """

    organization_id: Optional[str]
    """
    ID of Organization the rule is scoped to.
    
    One-of ('scope'): at most one of 'project_ids', 'organization_id' could be set.
    """


@dataclass
class SSHKey:
    """
    Ssh key.
    """

    id: str
    """
    ID of SSH key.
    """

    name: str
    """
    Name of SSH key.
    """

    public_key: str
    """
    Public key of SSH key.
    """

    fingerprint: str
    """
    Fingerprint of the SSH key.
    """

    created_at: Optional[datetime]
    """
    Creation date of SSH key.
    """

    updated_at: Optional[datetime]
    """
    Last update date of SSH key.
    """

    organization_id: str
    """
    ID of Organization linked to the SSH key.
    """

    project_id: str
    """
    ID of Project linked to the SSH key.
    """

    disabled: bool
    """
    SSH key status.
    """


@dataclass
class SetRulesResponse:
    """
    Set rules response.
    """

    rules: List[Rule]
    """
    Rules of the policy.
    """


@dataclass
class User:
    """
    User.
    """

    id: str
    """
    ID of user.
    """

    email: str
    """
    Email of user.
    """

    created_at: Optional[datetime]
    """
    Date user was created.
    """

    updated_at: Optional[datetime]
    """
    Date of last user update.
    """

    organization_id: str
    """
    ID of the Organization.
    """

    deletable: bool
    """
    Deletion status of user. Owners cannot be deleted.
    """

    last_login_at: Optional[datetime]
    """
    Date of the last login.
    """

    type_: UserType
    """
    Type of user.
    """

    two_factor_enabled: Optional[bool]
    """
    Deprecated, use "mfa" instead.
    :deprecated
    """

    status: UserStatus
    """
    Status of user invitation.
    """

    mfa: bool
    """
    Defines whether MFA is enabled.
    """

    account_root_user_id: str
    """
    ID of the account root user associated with the user.
    """


@dataclass
class ListSSHKeysRequest:
    order_by: Optional[ListSSHKeysRequestOrderBy]
    """
    Sort order of the SSH keys.
    """

    page: Optional[int]
    """
    Requested page number. Value must be greater or equal to 1.
    """

    page_size: Optional[int]
    """
    Number of items per page. Value must be between 1 and 100.
    """

    organization_id: Optional[str]
    """
    Filter by Organization ID.
    """

    name: Optional[str]
    """
    Name of group to find.
    """

    project_id: Optional[str]
    """
    Filter by Project ID.
    """

    disabled: Optional[bool]
    """
    Defines whether to include disabled SSH keys or not.
    """


@dataclass
class CreateSSHKeyRequest:
    name: Optional[str]
    """
    Name of the SSH key. Max length is 1000.
    """

    public_key: str
    """
    SSH public key. Currently only the ssh-rsa, ssh-dss (DSA), ssh-ed25519 and ecdsa keys with NIST curves are supported. Max length is 65000.
    """

    project_id: Optional[str]
    """
    Project the resource is attributed to.
    """


@dataclass
class GetSSHKeyRequest:
    ssh_key_id: str
    """
    ID of the SSH key.
    """


@dataclass
class UpdateSSHKeyRequest:
    ssh_key_id: str

    name: Optional[str]
    """
    Name of the SSH key. Max length is 1000.
    """

    disabled: Optional[bool]
    """
    Enable or disable the SSH key.
    """


@dataclass
class DeleteSSHKeyRequest:
    ssh_key_id: str


@dataclass
class ListUsersRequest:
    order_by: Optional[ListUsersRequestOrderBy]
    """
    Criteria for sorting results.
    """

    page_size: Optional[int]
    """
    Number of results per page. Value must be between 1 and 100.
    """

    page: Optional[int]
    """
    Page number. Value must be greater or equal to 1.
    """

    organization_id: Optional[str]
    """
    ID of the Organization to filter.
    """

    user_ids: Optional[List[str]]
    """
    Filter by list of IDs.
    """

    mfa: Optional[bool]
    """
    Filter by MFA status.
    """


@dataclass
class GetUserRequest:
    user_id: str
    """
    ID of the user to find.
    """


@dataclass
class DeleteUserRequest:
    user_id: str
    """
    ID of the user to delete.
    """


@dataclass
class CreateUserRequest:
    organization_id: Optional[str]
    """
    ID of the Organization.
    """

    email: str
    """
    Email of the user.
    """


@dataclass
class ListApplicationsRequest:
    order_by: Optional[ListApplicationsRequestOrderBy]
    """
    Criteria for sorting results.
    """

    page_size: Optional[int]
    """
    Number of results per page. Value must be between 1 and 100.
    """

    page: Optional[int]
    """
    Page number. Value must be greater than 1.
    """

    name: Optional[str]
    """
    Name of the application to filter.
    """

    organization_id: Optional[str]
    """
    ID of the Organization to filter.
    """

    editable: Optional[bool]
    """
    Defines whether to filter out editable applications or not.
    """

    application_ids: Optional[List[str]]
    """
    Filter by list of IDs.
    """


@dataclass
class CreateApplicationRequest:
    name: Optional[str]
    """
    Name of the application to create (max length is 64 characters).
    """

    organization_id: Optional[str]
    """
    ID of the Organization.
    """

    description: str
    """
    Description of the application (max length is 200 characters).
    """


@dataclass
class GetApplicationRequest:
    application_id: str
    """
    ID of the application to find.
    """


@dataclass
class UpdateApplicationRequest:
    application_id: str
    """
    ID of the application to update.
    """

    name: Optional[str]
    """
    New name for the application (max length is 64 chars).
    """

    description: Optional[str]
    """
    New description for the application (max length is 200 chars).
    """


@dataclass
class DeleteApplicationRequest:
    application_id: str
    """
    ID of the application to delete.
    """


@dataclass
class ListGroupsRequest:
    order_by: Optional[ListGroupsRequestOrderBy]
    """
    Sort order of groups.
    """

    page: Optional[int]
    """
    Requested page number. Value must be greater or equal to 1.
    """

    page_size: Optional[int]
    """
    Number of items per page. Value must be between 1 and 100.
    """

    organization_id: Optional[str]
    """
    Filter by Organization ID.
    """

    name: Optional[str]
    """
    Name of group to find.
    """

    application_ids: Optional[List[str]]
    """
    Filter by a list of application IDs.
    """

    user_ids: Optional[List[str]]
    """
    Filter by a list of user IDs.
    """

    group_ids: Optional[List[str]]
    """
    Filter by a list of group IDs.
    """


@dataclass
class CreateGroupRequest:
    organization_id: Optional[str]
    """
    ID of Organization linked to the group.
    """

    name: Optional[str]
    """
    Name of the group to create (max length is 64 chars). MUST be unique inside an Organization.
    """

    description: str
    """
    Description of the group to create (max length is 200 chars).
    """


@dataclass
class GetGroupRequest:
    group_id: str
    """
    ID of the group.
    """


@dataclass
class UpdateGroupRequest:
    group_id: str
    """
    ID of the group to update.
    """

    name: Optional[str]
    """
    New name for the group (max length is 64 chars). MUST be unique inside an Organization.
    """

    description: Optional[str]
    """
    New description for the group (max length is 200 chars).
    """


@dataclass
class SetGroupMembersRequest:
    group_id: str

    user_ids: List[str]

    application_ids: List[str]


@dataclass
class AddGroupMemberRequest:
    group_id: str
    """
    ID of the group.
    """

    user_id: Optional[str]
    """
    ID of the user to add.
    
    One-of ('member'): at most one of 'user_id', 'application_id' could be set.
    """

    application_id: Optional[str]
    """
    ID of the application to add.
    
    One-of ('member'): at most one of 'user_id', 'application_id' could be set.
    """


@dataclass
class AddGroupMembersRequest:
    group_id: str
    """
    ID of the group.
    """

    user_ids: Optional[List[str]]
    """
    IDs of the users to add.
    """

    application_ids: Optional[List[str]]
    """
    IDs of the applications to add.
    """


@dataclass
class RemoveGroupMemberRequest:
    group_id: str
    """
    ID of the group.
    """

    user_id: Optional[str]
    """
    ID of the user to remove.
    
    One-of ('member'): at most one of 'user_id', 'application_id' could be set.
    """

    application_id: Optional[str]
    """
    ID of the application to remove.
    
    One-of ('member'): at most one of 'user_id', 'application_id' could be set.
    """


@dataclass
class DeleteGroupRequest:
    group_id: str
    """
    ID of the group to delete.
    """


@dataclass
class ListPoliciesRequest:
    order_by: Optional[ListPoliciesRequestOrderBy]
    """
    Criteria for sorting results.
    """

    page_size: Optional[int]
    """
    Number of results per page. Value must be between 1 and 100.
    """

    page: Optional[int]
    """
    Page number. Value must be greater than 1.
    """

    organization_id: Optional[str]
    """
    ID of the Organization to filter.
    """

    editable: Optional[bool]
    """
    Defines whether or not filter out editable policies.
    """

    user_ids: Optional[List[str]]
    """
    Defines whether or not to filter by list of user IDs.
    """

    group_ids: Optional[List[str]]
    """
    Defines whether or not to filter by list of group IDs.
    """

    application_ids: Optional[List[str]]
    """
    Filter by a list of application IDs.
    """

    no_principal: Optional[bool]
    """
    Defines whether or not the policy is attributed to a principal.
    """

    policy_name: Optional[str]
    """
    Name of the policy to fetch.
    """


@dataclass
class CreatePolicyRequest:
    name: Optional[str]
    """
    Name of the policy to create (max length is 64 characters).
    """

    description: str
    """
    Description of the policy to create (max length is 200 characters).
    """

    organization_id: Optional[str]
    """
    ID of the Organization.
    """

    rules: Optional[List[RuleSpecs]]
    """
    Rules of the policy to create.
    """

    user_id: Optional[str]
    """
    ID of user attributed to the policy.
    
    One-of ('principal'): at most one of 'user_id', 'group_id', 'application_id', 'no_principal' could be set.
    """

    group_id: Optional[str]
    """
    ID of group attributed to the policy.
    
    One-of ('principal'): at most one of 'user_id', 'group_id', 'application_id', 'no_principal' could be set.
    """

    application_id: Optional[str]
    """
    ID of application attributed to the policy.
    
    One-of ('principal'): at most one of 'user_id', 'group_id', 'application_id', 'no_principal' could be set.
    """

    no_principal: Optional[bool]
    """
    Defines whether or not a policy is attributed to a principal.
    
    One-of ('principal'): at most one of 'user_id', 'group_id', 'application_id', 'no_principal' could be set.
    """


@dataclass
class GetPolicyRequest:
    policy_id: str
    """
    Id of policy to search.
    """


@dataclass
class UpdatePolicyRequest:
    policy_id: str
    """
    Id of policy to update.
    """

    name: Optional[str]
    """
    New name for the policy (max length is 64 characters).
    """

    description: Optional[str]
    """
    New description of policy (max length is 200 characters).
    """

    user_id: Optional[str]
    """
    New ID of user attributed to the policy.
    
    One-of ('principal'): at most one of 'user_id', 'group_id', 'application_id', 'no_principal' could be set.
    """

    group_id: Optional[str]
    """
    New ID of group attributed to the policy.
    
    One-of ('principal'): at most one of 'user_id', 'group_id', 'application_id', 'no_principal' could be set.
    """

    application_id: Optional[str]
    """
    New ID of application attributed to the policy.
    
    One-of ('principal'): at most one of 'user_id', 'group_id', 'application_id', 'no_principal' could be set.
    """

    no_principal: Optional[bool]
    """
    Defines whether or not the policy is attributed to a principal.
    
    One-of ('principal'): at most one of 'user_id', 'group_id', 'application_id', 'no_principal' could be set.
    """


@dataclass
class DeletePolicyRequest:
    policy_id: str
    """
    Id of policy to delete.
    """


@dataclass
class ClonePolicyRequest:
    policy_id: str


@dataclass
class SetRulesRequest:
    policy_id: str
    """
    Id of policy to update.
    """

    rules: List[RuleSpecs]
    """
    Rules of the policy to set.
    """


@dataclass
class ListRulesRequest:
    policy_id: str
    """
    Id of policy to search.
    """

    page_size: Optional[int]
    """
    Number of results per page. Value must be between 1 and 100.
    """

    page: Optional[int]
    """
    Page number. Value must be greater than 1.
    """


@dataclass
class ListPermissionSetsRequest:
    order_by: Optional[ListPermissionSetsRequestOrderBy]
    """
    Criteria for sorting results.
    """

    page_size: Optional[int]
    """
    Number of results per page. Value must be between 1 and 100.
    """

    page: Optional[int]
    """
    Page number. Value must be greater than 1.
    """

    organization_id: Optional[str]
    """
    Filter by Organization ID.
    """


@dataclass
class ListAPIKeysRequest:
    order_by: Optional[ListAPIKeysRequestOrderBy]
    """
    Criteria for sorting results.
    """

    page: Optional[int]
    """
    Page number. Value must be greater or equal to 1.
    """

    page_size: Optional[int]
    """
    Number of results per page. Value must be between 1 and 100.
    """

    organization_id: Optional[str]
    """
    ID of Organization.
    """

    application_id: Optional[str]
    """
    ID of application that bears the API key.
    
    One-of ('bearer'): at most one of 'application_id', 'user_id' could be set.
    :deprecated
    """

    user_id: Optional[str]
    """
    ID of user that bears the API key.
    
    One-of ('bearer'): at most one of 'application_id', 'user_id' could be set.
    :deprecated
    """

    editable: Optional[bool]
    """
    Defines whether to filter out editable API keys or not.
    """

    expired: Optional[bool]
    """
    Defines whether to filter out expired API keys or not.
    """

    access_key: Optional[str]
    """
    Filter by access key.
    """

    description: Optional[str]
    """
    Filter by description.
    """

    bearer_id: Optional[str]
    """
    Filter by bearer ID.
    """

    bearer_type: Optional[BearerType]
    """
    Filter by type of bearer.
    """


@dataclass
class CreateAPIKeyRequest:
    application_id: Optional[str]
    """
    ID of the application.
    
    One-of ('bearer'): at most one of 'application_id', 'user_id' could be set.
    """

    user_id: Optional[str]
    """
    ID of the user.
    
    One-of ('bearer'): at most one of 'application_id', 'user_id' could be set.
    """

    expires_at: Optional[datetime]
    """
    Expiration date of the API key.
    """

    default_project_id: Optional[str]
    """
    Default Project ID to use with Object Storage.
    """

    description: str
    """
    Description of the API key (max length is 200 characters).
    """


@dataclass
class GetAPIKeyRequest:
    access_key: str
    """
    Access key to search for.
    """


@dataclass
class UpdateAPIKeyRequest:
    access_key: str
    """
    Access key to update.
    """

    default_project_id: Optional[str]
    """
    New default Project ID to set.
    """

    description: Optional[str]
    """
    New description to update.
    """


@dataclass
class DeleteAPIKeyRequest:
    access_key: str
    """
    Access key to delete.
    """


@dataclass
class ListQuotaRequest:
    order_by: Optional[ListQuotaRequestOrderBy]
    """
    Criteria for sorting results.
    """

    page_size: Optional[int]
    """
    Number of results per page. Value must be between 1 and 100.
    """

    page: Optional[int]
    """
    Page number. Value must be greater than 1.
    """

    organization_id: Optional[str]
    """
    Filter by Organization ID.
    """


@dataclass
class GetQuotumRequest:
    quotum_name: str
    """
    Name of the quota to get.
    """

    organization_id: Optional[str]
    """
    ID of the Organization.
    """


@dataclass
class ListJWTsRequest:
    order_by: Optional[ListJWTsRequestOrderBy]
    """
    Criteria for sorting results.
    """

    audience_id: str
    """
    ID of the user to search.
    """

    page_size: Optional[int]
    """
    Number of results per page. Value must be between 1 and 100.
    """

    page: Optional[int]
    """
    Page number. Value must be greater to 1.
    """

    expired: Optional[bool]
    """
    Filter out expired JWTs or not.
    """


@dataclass
class GetJWTRequest:
    jti: str
    """
    JWT ID of the JWT to get.
    """


@dataclass
class DeleteJWTRequest:
    jti: str
    """
    JWT ID of the JWT to delete.
    """



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from typing import List

from .types import (
    DNSZoneStatus,
    DomainFeatureStatus,
    DomainRegistrationStatusTransferStatus,
    DomainStatus,
    HostStatus,
    SSLCertificateStatus,
    TaskStatus,
)


DNS_ZONE_TRANSIENT_STATUSES: List[DNSZoneStatus] = [
    DNSZoneStatus.PENDING,
]
"""
Lists transient statutes of the enum :class:`DNSZoneStatus <DNSZoneStatus>`.
"""

DOMAIN_FEATURE_TRANSIENT_STATUSES: List[DomainFeatureStatus] = [
    DomainFeatureStatus.ENABLING,
    DomainFeatureStatus.DISABLING,
]
"""
Lists transient statutes of the enum :class:`DomainFeatureStatus <DomainFeatureStatus>`.
"""

DOMAIN_REGISTRATION_STATUS_TRANSFER_TRANSIENT_STATUSES: List[
    DomainRegistrationStatusTransferStatus
] = [
    DomainRegistrationStatusTransferStatus.PENDING,
    DomainRegistrationStatusTransferStatus.PROCESSING,
]
"""
Lists transient statutes of the enum :class:`DomainRegistrationStatusTransferStatus <DomainRegistrationStatusTransferStatus>`.
"""

DOMAIN_TRANSIENT_STATUSES: List[DomainStatus] = [
    DomainStatus.CREATING,
    DomainStatus.RENEWING,
    DomainStatus.XFERING,
    DomainStatus.EXPIRING,
    DomainStatus.UPDATING,
    DomainStatus.CHECKING,
    DomainStatus.DELETING,
]
"""
Lists transient statutes of the enum :class:`DomainStatus <DomainStatus>`.
"""

HOST_TRANSIENT_STATUSES: List[HostStatus] = [
    HostStatus.UPDATING,
    HostStatus.DELETING,
]
"""
Lists transient statutes of the enum :class:`HostStatus <HostStatus>`.
"""

SSL_CERTIFICATE_TRANSIENT_STATUSES: List[SSLCertificateStatus] = [
    SSLCertificateStatus.PENDING,
]
"""
Lists transient statutes of the enum :class:`SSLCertificateStatus <SSLCertificateStatus>`.
"""

TASK_TRANSIENT_STATUSES: List[TaskStatus] = [
    TaskStatus.PENDING,
]
"""
Lists transient statutes of the enum :class:`TaskStatus <TaskStatus>`.
"""



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from dateutil import parser
from .types import (
    MACAddressType,
    AttachFlexibleIPsResponse,
    DetachFlexibleIPsResponse,
    FlexibleIP,
    ListFlexibleIPsResponse,
    MACAddress,
    CreateFlexibleIPRequest,
    UpdateFlexibleIPRequest,
    AttachFlexibleIPRequest,
    DetachFlexibleIPRequest,
    GenerateMACAddrRequest,
    DuplicateMACAddrRequest,
    MoveMACAddrRequest,
)


def unmarshal_MACAddress(data: Any) -> MACAddress:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'MACAddress' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("mac_address", None)
    args["mac_address"] = field

    field = data.get("mac_type", None)
    args["mac_type"] = field

    field = data.get("status", None)
    args["status"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("zone", None)
    args["zone"] = field

    return MACAddress(**args)


def unmarshal_FlexibleIP(data: Any) -> FlexibleIP:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'FlexibleIP' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at", None)
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("description", None)
    args["description"] = field

    field = data.get("id", None)
    args["id"] = field

    field = data.get("ip_address", None)
    args["ip_address"] = field

    field = data.get("mac_address", None)
    args["mac_address"] = unmarshal_MACAddress(field) if field is not None else None

    field = data.get("organization_id", None)
    args["organization_id"] = field

    field = data.get("project_id", None)
    args["project_id"] = field

    field = data.get("reverse", None)
    args["reverse"] = field

    field = data.get("server_id", None)
    args["server_id"] = field

    field = data.get("status", None)
    args["status"] = field

    field = data.get("tags", None)
    args["tags"] = field

    field = data.get("updated_at", None)
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("zone", None)
    args["zone"] = field

    return FlexibleIP(**args)


def unmarshal_AttachFlexibleIPsResponse(data: Any) -> AttachFlexibleIPsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'AttachFlexibleIPsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("flexible_ips", None)
    args["flexible_ips"] = (
        [unmarshal_FlexibleIP(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return AttachFlexibleIPsResponse(**args)


def unmarshal_DetachFlexibleIPsResponse(data: Any) -> DetachFlexibleIPsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'DetachFlexibleIPsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("flexible_ips", None)
    args["flexible_ips"] = (
        [unmarshal_FlexibleIP(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return DetachFlexibleIPsResponse(**args)


def unmarshal_ListFlexibleIPsResponse(data: Any) -> ListFlexibleIPsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListFlexibleIPsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("flexible_ips", None)
    args["flexible_ips"] = (
        [unmarshal_FlexibleIP(v) for v in field] if field is not None else None
    )

    field = data.get("total_count", None)
    args["total_count"] = field

    return ListFlexibleIPsResponse(**args)


def marshal_AttachFlexibleIPRequest(
    request: AttachFlexibleIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.fips_ids is not None:
        output["fips_ids"] = request.fips_ids

    if request.server_id is not None:
        output["server_id"] = request.server_id

    return output


def marshal_CreateFlexibleIPRequest(
    request: CreateFlexibleIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.description is not None:
        output["description"] = request.description

    if request.is_ipv6 is not None:
        output["is_ipv6"] = request.is_ipv6

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.reverse is not None:
        output["reverse"] = request.reverse

    if request.server_id is not None:
        output["server_id"] = request.server_id

    if request.tags is not None:
        output["tags"] = request.tags

    return output


def marshal_DetachFlexibleIPRequest(
    request: DetachFlexibleIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.fips_ids is not None:
        output["fips_ids"] = request.fips_ids

    return output


def marshal_DuplicateMACAddrRequest(
    request: DuplicateMACAddrRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.duplicate_from_fip_id is not None:
        output["duplicate_from_fip_id"] = request.duplicate_from_fip_id

    return output


def marshal_GenerateMACAddrRequest(
    request: GenerateMACAddrRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.mac_type is not None:
        output["mac_type"] = MACAddressType(request.mac_type)

    return output


def marshal_MoveMACAddrRequest(
    request: MoveMACAddrRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.dst_fip_id is not None:
        output["dst_fip_id"] = request.dst_fip_id

    return output


def marshal_UpdateFlexibleIPRequest(
    request: UpdateFlexibleIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.description is not None:
        output["description"] = request.description

    if request.reverse is not None:
        output["reverse"] = request.reverse

    if request.tags is not None:
        output["tags"] = request.tags

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import List, Optional

from scaleway_core.api import API
from scaleway_core.utils import (
    fetch_all_pages_async,
    validate_path_param,
)
from .types import (
    GetImageResponse,
    GetVersionResponse,
    Image,
    ListImagesResponse,
    ListVersionsResponse,
)
from .marshalling import (
    unmarshal_GetImageResponse,
    unmarshal_GetVersionResponse,
    unmarshal_ListImagesResponse,
    unmarshal_ListVersionsResponse,
)


class MarketplaceV1API(API):
    """
    Marketplace API.

    Marketplace API.
    """

    async def list_images(
        self,
        *,
        per_page: Optional[int] = None,
        page: Optional[int] = None,
    ) -> ListImagesResponse:
        """
        List marketplace images.
        :param per_page: A positive integer lower or equal to 100 to select the number of items to display.
        :param page: A positive integer to choose the page to display.
        :return: :class:`ListImagesResponse <ListImagesResponse>`

        Usage:
        ::

            result = await api.list_images()
        """

        res = self._request(
            "GET",
            f"/marketplace/v1/images",
            params={
                "page": page,
                "per_page": per_page or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListImagesResponse(res.json())

    async def list_images_all(
        self,
        *,
        per_page: Optional[int] = None,
        page: Optional[int] = None,
    ) -> List[Image]:
        """
        List marketplace images.
        :param per_page: A positive integer lower or equal to 100 to select the number of items to display.
        :param page: A positive integer to choose the page to display.
        :return: :class:`List[ListImagesResponse] <List[ListImagesResponse]>`

        Usage:
        ::

            result = await api.list_images_all()
        """

        return await fetch_all_pages_async(
            type=ListImagesResponse,
            key="images",
            fetcher=self.list_images,
            args={
                "per_page": per_page,
                "page": page,
            },
        )

    async def get_image(
        self,
        *,
        image_id: str,
    ) -> GetImageResponse:
        """
        Get a specific marketplace image.
        :param image_id: Display the image name.
        :return: :class:`GetImageResponse <GetImageResponse>`

        Usage:
        ::

            result = await api.get_image(image_id="example")
        """

        param_image_id = validate_path_param("image_id", image_id)

        res = self._request(
            "GET",
            f"/marketplace/v1/images/{param_image_id}",
        )

        self._throw_on_error(res)
        return unmarshal_GetImageResponse(res.json())

    async def list_versions(
        self,
        *,
        image_id: str,
    ) -> ListVersionsResponse:
        """

        Usage:
        ::

            result = await api.list_versions(image_id="example")
        """

        param_image_id = validate_path_param("image_id", image_id)

        res = self._request(
            "GET",
            f"/marketplace/v1/images/{param_image_id}/versions",
        )

        self._throw_on_error(res)
        return unmarshal_ListVersionsResponse(res.json())

    async def get_version(
        self,
        *,
        image_id: str,
        version_id: str,
    ) -> GetVersionResponse:
        """

        Usage:
        ::

            result = await api.get_version(
                image_id="example",
                version_id="example",
            )
        """

        param_image_id = validate_path_param("image_id", image_id)
        param_version_id = validate_path_param("version_id", version_id)

        res = self._request(
            "GET",
            f"/marketplace/v1/images/{param_image_id}/versions/{param_version_id}",
        )

        self._throw_on_error(res)
        return unmarshal_GetVersionResponse(res.json())



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import BearerType
from .types import ListAPIKeysRequestOrderBy
from .types import ListApplicationsRequestOrderBy
from .types import ListGroupsRequestOrderBy
from .types import ListJWTsRequestOrderBy
from .types import ListPermissionSetsRequestOrderBy
from .types import ListPoliciesRequestOrderBy
from .types import ListQuotaRequestOrderBy
from .types import ListSSHKeysRequestOrderBy
from .types import ListUsersRequestOrderBy
from .types import PermissionSetScopeType
from .types import UserStatus
from .types import UserType
from .types import APIKey
from .types import Application
from .types import Group
from .types import JWT
from .types import ListAPIKeysResponse
from .types import ListApplicationsResponse
from .types import ListGroupsResponse
from .types import ListJWTsResponse
from .types import ListPermissionSetsResponse
from .types import ListPoliciesResponse
from .types import ListQuotaResponse
from .types import ListRulesResponse
from .types import ListSSHKeysResponse
from .types import ListUsersResponse
from .types import PermissionSet
from .types import Policy
from .types import Quotum
from .types import Rule
from .types import RuleSpecs
from .types import SSHKey
from .types import SetRulesResponse
from .types import User
from .api import IamV1Alpha1API

__all__ = [
    "BearerType",
    "ListAPIKeysRequestOrderBy",
    "ListApplicationsRequestOrderBy",
    "ListGroupsRequestOrderBy",
    "ListJWTsRequestOrderBy",
    "ListPermissionSetsRequestOrderBy",
    "ListPoliciesRequestOrderBy",
    "ListQuotaRequestOrderBy",
    "ListSSHKeysRequestOrderBy",
    "ListUsersRequestOrderBy",
    "PermissionSetScopeType",
    "UserStatus",
    "UserType",
    "APIKey",
    "Application",
    "Group",
    "JWT",
    "ListAPIKeysResponse",
    "ListApplicationsResponse",
    "ListGroupsResponse",
    "ListJWTsResponse",
    "ListPermissionSetsResponse",
    "ListPoliciesResponse",
    "ListQuotaResponse",
    "ListRulesResponse",
    "ListSSHKeysResponse",
    "ListUsersResponse",
    "PermissionSet",
    "Policy",
    "Quotum",
    "Rule",
    "RuleSpecs",
    "SSHKey",
    "SetRulesResponse",
    "User",
    "IamV1Alpha1API",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional

from scaleway_core.bridge import (
    Region,
)


class ListSecretsRequestOrderBy(str, Enum):
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    UPDATED_AT_ASC = "updated_at_asc"
    UPDATED_AT_DESC = "updated_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class Product(str, Enum):
    UNKNOWN = "unknown"

    def __str__(self) -> str:
        return str(self.value)


class SecretStatus(str, Enum):
    READY = "ready"
    LOCKED = "locked"

    def __str__(self) -> str:
        return str(self.value)


class SecretType(str, Enum):
    UNKNOWN_SECRET_TYPE = "unknown_secret_type"
    OPAQUE = "opaque"
    CERTIFICATE = "certificate"

    def __str__(self) -> str:
        return str(self.value)


class SecretVersionStatus(str, Enum):
    UNKNOWN = "unknown"
    ENABLED = "enabled"
    DISABLED = "disabled"
    DESTROYED = "destroyed"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class AccessSecretVersionResponse:
    """
    Access secret version response.
    """

    secret_id: str
    """
    ID of the secret.
    """

    revision: int
    """
    Version number.
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1.
    """

    data: str
    """
    The base64-encoded secret payload of the version.
    """

    data_crc32: Optional[int]
    """
    The CRC32 checksum of the data as a base-10 integer.
    This field is only available if a CRC32 was supplied during the creation of the version.
    """


@dataclass
class ListSecretVersionsResponse:
    """
    List secret versions response.
    """

    versions: List[SecretVersion]
    """
    Single page of versions.
    """

    total_count: int
    """
    Number of versions.
    """


@dataclass
class ListSecretsResponse:
    """
    List secrets response.
    """

    secrets: List[Secret]
    """
    Single page of secrets matching the requested criteria.
    """

    total_count: int
    """
    Count of all secrets matching the requested criteria.
    """


@dataclass
class ListTagsResponse:
    """
    List tags response.
    """

    tags: List[str]
    """
    List of tags.
    """

    total_count: int
    """
    Count of all tags matching the requested criteria.
    """


@dataclass
class PasswordGenerationParams:
    """
    Password generation params.
    """

    length: int
    """
    Length of the password to generate (between 1 and 1024).
    """

    no_lowercase_letters: bool
    """
    Do not include lower case letters by default in the alphabet.
    """

    no_uppercase_letters: bool
    """
    Do not include upper case letters by default in the alphabet.
    """

    no_digits: bool
    """
    Do not include digits by default in the alphabet.
    """

    additional_chars: str
    """
    Additional ascii characters to be included in the alphabet.
    """


@dataclass
class Secret:
    """
    Secret.
    """

    id: str
    """
    ID of the secret.
    """

    project_id: str
    """
    ID of the Project containing the secret.
    """

    name: str
    """
    Name of the secret.
    """

    status: SecretStatus
    """
    Current status of the secret.
    * `ready`: the secret can be read, modified and deleted.
    * `locked`: no action can be performed on the secret. This status can only be applied and removed by Scaleway.
    """

    created_at: Optional[datetime]
    """
    Date and time of the secret's creation.
    """

    updated_at: Optional[datetime]
    """
    Last update of the secret.
    """

    tags: List[str]
    """
    List of the secret's tags.
    """

    version_count: int
    """
    Number of versions for this secret.
    """

    description: Optional[str]
    """
    Updated description of the secret.
    """

    is_managed: bool
    """
    Returns `true` for secrets that are managed by another product.
    """

    is_protected: bool
    """
    Returns `true` for protected secrets that cannot be deleted.
    """

    type_: SecretType
    """
    Type of the secret.
    See `Secret.Type` enum for description of values.
    """

    region: Region
    """
    Region of the secret.
    """


@dataclass
class SecretVersion:
    """
    Secret version.
    """

    revision: int
    """
    Version number.
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1.
    """

    secret_id: str
    """
    ID of the secret.
    """

    status: SecretVersionStatus
    """
    Current status of the version.
    * `unknown`: the version is in an invalid state.
    * `enabled`: the version is accessible.
    * `disabled`: the version is not accessible but can be enabled.
    * `destroyed`: the version is permanently deleted. It is not possible to recover it.
    """

    created_at: Optional[datetime]
    """
    Date and time of the version's creation.
    """

    updated_at: Optional[datetime]
    """
    Last update of the version.
    """

    description: Optional[str]
    """
    Description of the version.
    """

    is_latest: bool
    """
    Returns `true` if the version is the latest.
    """


@dataclass
class CreateSecretRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    project_id: Optional[str]
    """
    ID of the Project containing the secret.
    """

    name: str
    """
    Name of the secret.
    """

    tags: Optional[List[str]]
    """
    List of the secret's tags.
    """

    description: Optional[str]
    """
    Description of the secret.
    """

    type_: SecretType
    """
    Type of the secret.
    (Optional.) See `Secret.Type` enum for description of values. If not specified, the type is `Opaque`.
    """


@dataclass
class GetSecretRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret.
    """


@dataclass
class GetSecretByNameRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_name: str
    """
    Name of the secret.
    """

    project_id: Optional[str]
    """
    ID of the Project to target.
    (Optional.) If not specified, Secret Manager will look for the secret in all Projects.
    """


@dataclass
class UpdateSecretRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret.
    """

    name: Optional[str]
    """
    Secret's updated name (optional).
    """

    tags: Optional[List[str]]
    """
    Secret's updated list of tags (optional).
    """

    description: Optional[str]
    """
    Description of the secret.
    """


@dataclass
class ListSecretsRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    organization_id: Optional[str]
    """
    Filter by Organization ID (optional).
    """

    project_id: Optional[str]
    """
    Filter by Project ID (optional).
    """

    order_by: Optional[ListSecretsRequestOrderBy]

    page: Optional[int]

    page_size: Optional[int]

    tags: Optional[List[str]]
    """
    List of tags to filter on (optional).
    """

    name: Optional[str]
    """
    Filter by secret name (optional).
    """

    is_managed: Optional[bool]
    """
    Filter by managed / not managed (optional).
    """


@dataclass
class DeleteSecretRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret.
    """


@dataclass
class ProtectSecretRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret to protect.
    """


@dataclass
class UnprotectSecretRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret to unprotect.
    """


@dataclass
class AddSecretOwnerRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret.
    """

    product_name: Optional[str]
    """
    (Deprecated: use `product` field) Name of the product to add.
    :deprecated
    """

    product: Product
    """
    ID of the product to add.
    See `Product` enum for description of values.
    """


@dataclass
class CreateSecretVersionRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret.
    """

    data: str
    """
    The base64-encoded secret payload of the version.
    """

    description: Optional[str]
    """
    Description of the version.
    """

    disable_previous: Optional[bool]
    """
    Disable the previous secret version.
    (Optional.) If there is no previous version or if the previous version was already disabled, does nothing.
    """

    password_generation: Optional[PasswordGenerationParams]
    """
    Options to generate a password.
    (Optional.) If specified, a random password will be generated. The `data` and `data_crc32` fields must be empty. By default, the generator will use upper and lower case letters, and digits. This behavior can be tuned using the generation parameters.
    
    One-of ('_password_generation'): at most one of 'password_generation' could be set.
    :deprecated
    """

    data_crc32: Optional[int]
    """
    (Optional.) The CRC32 checksum of the data as a base-10 integer.
    If specified, Secret Manager will verify the integrity of the data received against the given CRC32 checksum. An error is returned if the CRC32 does not match. If, however, the CRC32 matches, it will be stored and returned along with the SecretVersion on future access requests.
    """


@dataclass
class GeneratePasswordRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret.
    """

    description: Optional[str]
    """
    Description of the version.
    """

    disable_previous: Optional[bool]
    """
    (Optional.) Disable the previous secret version.
    This has no effect if there is no previous version or if the previous version was already disabled.
    """

    length: int
    """
    Length of the password to generate (between 1 and 1024 characters).
    """

    no_lowercase_letters: Optional[bool]
    """
    (Optional.) Exclude lower case letters by default in the password character set.
    """

    no_uppercase_letters: Optional[bool]
    """
    (Optional.) Exclude upper case letters by default in the password character set.
    """

    no_digits: Optional[bool]
    """
    (Optional.) Exclude digits by default in the password character set.
    """

    additional_chars: Optional[str]
    """
    (Optional.) Additional ASCII characters to be included in the password character set.
    """


@dataclass
class GetSecretVersionRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret.
    """

    revision: str
    """
    Version number.
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be a number or "latest".
    """


@dataclass
class GetSecretVersionByNameRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_name: str
    """
    Name of the secret.
    """

    revision: str
    """
    Version number.
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be a number or "latest".
    """

    project_id: Optional[str]
    """
    ID of the Project to target.
    (Optional.) If not specified, Secret Manager will look for the secret version in all Projects.
    """


@dataclass
class UpdateSecretVersionRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret.
    """

    revision: str
    """
    Version number.
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be a number or "latest".
    """

    description: Optional[str]
    """
    Description of the version.
    """


@dataclass
class ListSecretVersionsRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret.
    """

    page: Optional[int]

    page_size: Optional[int]

    status: Optional[List[SecretVersionStatus]]
    """
    Filter results by status.
    """


@dataclass
class ListSecretVersionsByNameRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_name: str
    """
    Name of the secret.
    """

    page: Optional[int]

    page_size: Optional[int]

    status: Optional[List[SecretVersionStatus]]
    """
    Filter results by status.
    """

    project_id: Optional[str]
    """
    ID of the Project to target.
    (Optional.) If not specified, Secret Manager will look for the secret in all Projects.
    """


@dataclass
class EnableSecretVersionRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret.
    """

    revision: str
    """
    Version number.
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be a number or "latest".
    """


@dataclass
class DisableSecretVersionRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret.
    """

    revision: str
    """
    Version number.
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be a number or "latest".
    """


@dataclass
class AccessSecretVersionRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret.
    """

    revision: str
    """
    Version number.
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be a number or "latest".
    """


@dataclass
class AccessSecretVersionByNameRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_name: str
    """
    Name of the secret.
    """

    revision: str
    """
    Version number.
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be a number or "latest".
    """

    project_id: Optional[str]
    """
    ID of the Project to target.
    (Optional.) If not specified, Secret Manager will look for the secret version in all Projects.
    """


@dataclass
class DestroySecretVersionRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    secret_id: str
    """
    ID of the secret.
    """

    revision: str
    """
    Version number.
    The first version of the secret is numbered 1, and all subsequent revisions augment by 1. Value can be a number or "latest".
    """


@dataclass
class ListTagsRequest:
    region: Optional[Region]
    """
    Region to target. If none is passed will use default region from the config.
    """

    project_id: Optional[str]
    """
    ID of the Project to target.
    (Optional.) If not specified, Secret Manager will look for tags in all Projects.
    """

    page: Optional[int]

    page_size: Optional[int]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import AutoscalerEstimator
from .types import AutoscalerExpander
from .types import CNI
from .types import ClusterStatus
from .types import ClusterTypeAvailability
from .types import ClusterTypeResiliency
from .types import Ingress
from .types import ListClustersRequestOrderBy
from .types import ListNodesRequestOrderBy
from .types import ListPoolsRequestOrderBy
from .types import MaintenanceWindowDayOfTheWeek
from .types import NodeStatus
from .types import PoolStatus
from .types import PoolVolumeType
from .types import Runtime
from .types import Cluster
from .types import ClusterAutoUpgrade
from .types import ClusterAutoscalerConfig
from .types import ClusterOpenIDConnectConfig
from .types import ClusterType
from .types import CreateClusterRequestAutoUpgrade
from .types import CreateClusterRequestAutoscalerConfig
from .types import CreateClusterRequestOpenIDConnectConfig
from .types import CreateClusterRequestPoolConfig
from .types import CreateClusterRequestPoolConfigUpgradePolicy
from .types import CreatePoolRequestUpgradePolicy
from .types import ExternalNode
from .types import ListClusterAvailableTypesResponse
from .types import ListClusterAvailableVersionsResponse
from .types import ListClusterTypesResponse
from .types import ListClustersResponse
from .types import ListNodesResponse
from .types import ListPoolsResponse
from .types import ListVersionsResponse
from .types import MaintenanceWindow
from .types import Node
from .types import Pool
from .types import PoolUpgradePolicy
from .types import UpdateClusterRequestAutoUpgrade
from .types import UpdateClusterRequestAutoscalerConfig
from .types import UpdateClusterRequestOpenIDConnectConfig
from .types import UpdatePoolRequestUpgradePolicy
from .types import Version
from .content import CLUSTER_TRANSIENT_STATUSES
from .content import NODE_TRANSIENT_STATUSES
from .content import POOL_TRANSIENT_STATUSES
from .api import K8SV1API

__all__ = [
    "AutoscalerEstimator",
    "AutoscalerExpander",
    "CNI",
    "ClusterStatus",
    "ClusterTypeAvailability",
    "ClusterTypeResiliency",
    "Ingress",
    "ListClustersRequestOrderBy",
    "ListNodesRequestOrderBy",
    "ListPoolsRequestOrderBy",
    "MaintenanceWindowDayOfTheWeek",
    "NodeStatus",
    "PoolStatus",
    "PoolVolumeType",
    "Runtime",
    "Cluster",
    "ClusterAutoUpgrade",
    "ClusterAutoscalerConfig",
    "ClusterOpenIDConnectConfig",
    "ClusterType",
    "CreateClusterRequestAutoUpgrade",
    "CreateClusterRequestAutoscalerConfig",
    "CreateClusterRequestOpenIDConnectConfig",
    "CreateClusterRequestPoolConfig",
    "CreateClusterRequestPoolConfigUpgradePolicy",
    "CreatePoolRequestUpgradePolicy",
    "ExternalNode",
    "ListClusterAvailableTypesResponse",
    "ListClusterAvailableVersionsResponse",
    "ListClusterTypesResponse",
    "ListClustersResponse",
    "ListNodesResponse",
    "ListPoolsResponse",
    "ListVersionsResponse",
    "MaintenanceWindow",
    "Node",
    "Pool",
    "PoolUpgradePolicy",
    "UpdateClusterRequestAutoUpgrade",
    "UpdateClusterRequestAutoscalerConfig",
    "UpdateClusterRequestOpenIDConnectConfig",
    "UpdatePoolRequestUpgradePolicy",
    "Version",
    "CLUSTER_TRANSIENT_STATUSES",
    "NODE_TRANSIENT_STATUSES",
    "POOL_TRANSIENT_STATUSES",
    "K8SV1API",
]



"""Scaleway SDK for Python - Async"""

import pkg_resources

__version__: str = pkg_resources.get_distribution(__name__).version

from scaleway_core.api import (
    API,
    ScalewayException,
)

from scaleway_core.client import Client

from scaleway_core.profile import (
    Profile,
    ProfileConfig,
    ProfileDefaults,
)

from scaleway_core.utils.waiter import (
    WaitForOptions,
    WaitForStopCondition,
)

from scaleway_core.bridge import (
    Money,
    Region,
    ALL_REGIONS,
    Zone,
    ALL_ZONES,
    ScwFile,
    ServiceInfo,
    TimeSeriesPoint,
    TimeSeries,
)

__all__ = [
    "API",
    "ScalewayException",
    "Client",
    "Profile",
    "ProfileConfig",
    "ProfileDefaults",
    "WaitForOptions",
    "WaitForStopCondition",
    "Money",
    "Region",
    "ALL_REGIONS",
    "Zone",
    "ALL_ZONES",
    "ScwFile",
    "ServiceInfo",
    "TimeSeriesPoint",
    "TimeSeries",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from typing import List

from .types import (
    ImageStatus,
    NamespaceStatus,
    TagStatus,
)


IMAGE_TRANSIENT_STATUSES: List[ImageStatus] = [
    ImageStatus.DELETING,
]
"""
Lists transient statutes of the enum :class:`ImageStatus <ImageStatus>`.
"""

NAMESPACE_TRANSIENT_STATUSES: List[NamespaceStatus] = [
    NamespaceStatus.DELETING,
]
"""
Lists transient statutes of the enum :class:`NamespaceStatus <NamespaceStatus>`.
"""

TAG_TRANSIENT_STATUSES: List[TagStatus] = [
    TagStatus.DELETING,
]
"""
Lists transient statutes of the enum :class:`TagStatus <TagStatus>`.
"""



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import CronStatus
from .types import DomainStatus
from .types import FunctionHttpOption
from .types import FunctionPrivacy
from .types import FunctionRuntime
from .types import FunctionStatus
from .types import ListCronsRequestOrderBy
from .types import ListDomainsRequestOrderBy
from .types import ListFunctionsRequestOrderBy
from .types import ListLogsRequestOrderBy
from .types import ListNamespacesRequestOrderBy
from .types import ListTokensRequestOrderBy
from .types import ListTriggersRequestOrderBy
from .types import LogStream
from .types import NamespaceStatus
from .types import NullValue
from .types import RuntimeStatus
from .types import TokenStatus
from .types import TriggerInputType
from .types import TriggerStatus
from .types import CreateTriggerRequestMnqNatsClientConfig
from .types import CreateTriggerRequestMnqSqsClientConfig
from .types import CreateTriggerRequestSqsClientConfig
from .types import Cron
from .types import Domain
from .types import DownloadURL
from .types import Function
from .types import ListCronsResponse
from .types import ListDomainsResponse
from .types import ListFunctionRuntimesResponse
from .types import ListFunctionsResponse
from .types import ListLogsResponse
from .types import ListNamespacesResponse
from .types import ListTokensResponse
from .types import ListTriggersResponse
from .types import Log
from .types import Namespace
from .types import Runtime
from .types import Secret
from .types import SecretHashedValue
from .types import Token
from .types import Trigger
from .types import TriggerMnqNatsClientConfig
from .types import TriggerMnqSqsClientConfig
from .types import TriggerSqsClientConfig
from .types import UpdateTriggerRequestSqsClientConfig
from .types import UploadURL
from .content import CRON_TRANSIENT_STATUSES
from .content import DOMAIN_TRANSIENT_STATUSES
from .content import FUNCTION_TRANSIENT_STATUSES
from .content import NAMESPACE_TRANSIENT_STATUSES
from .content import TOKEN_TRANSIENT_STATUSES
from .content import TRIGGER_TRANSIENT_STATUSES
from .api import FunctionV1Beta1API

__all__ = [
    "CronStatus",
    "DomainStatus",
    "FunctionHttpOption",
    "FunctionPrivacy",
    "FunctionRuntime",
    "FunctionStatus",
    "ListCronsRequestOrderBy",
    "ListDomainsRequestOrderBy",
    "ListFunctionsRequestOrderBy",
    "ListLogsRequestOrderBy",
    "ListNamespacesRequestOrderBy",
    "ListTokensRequestOrderBy",
    "ListTriggersRequestOrderBy",
    "LogStream",
    "NamespaceStatus",
    "NullValue",
    "RuntimeStatus",
    "TokenStatus",
    "TriggerInputType",
    "TriggerStatus",
    "CreateTriggerRequestMnqNatsClientConfig",
    "CreateTriggerRequestMnqSqsClientConfig",
    "CreateTriggerRequestSqsClientConfig",
    "Cron",
    "Domain",
    "DownloadURL",
    "Function",
    "ListCronsResponse",
    "ListDomainsResponse",
    "ListFunctionRuntimesResponse",
    "ListFunctionsResponse",
    "ListLogsResponse",
    "ListNamespacesResponse",
    "ListTokensResponse",
    "ListTriggersResponse",
    "Log",
    "Namespace",
    "Runtime",
    "Secret",
    "SecretHashedValue",
    "Token",
    "Trigger",
    "TriggerMnqNatsClientConfig",
    "TriggerMnqSqsClientConfig",
    "TriggerSqsClientConfig",
    "UpdateTriggerRequestSqsClientConfig",
    "UploadURL",
    "CRON_TRANSIENT_STATUSES",
    "DOMAIN_TRANSIENT_STATUSES",
    "FUNCTION_TRANSIENT_STATUSES",
    "NAMESPACE_TRANSIENT_STATUSES",
    "TOKEN_TRANSIENT_STATUSES",
    "TRIGGER_TRANSIENT_STATUSES",
    "FunctionV1Beta1API",
]
